/* AxMl, a program for estimation of phylogenetic trees from sequences 
 * implementing SEVs (Subtree Equality Vectors) Copyright (C) 2002 by
 * Alexandros P. Stamatakis.
 *  Derived from:
 *  fastDNAml, a program for estimation of phylogenetic trees from sequences.
 *  Copyright (C) 1998, 1999, 2000 by Gary J. Olsen
 *
 *  This program is free software; you may redistribute it and/or modify its
 *  under the terms of the GNU General Public License as published by the Free
 *  Software Foundation; either version 2 of the License, or (at your option)
 *  any later version.
 *
 *  This program is distributed in the hope that it will be useful, but
 *  WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
 *  or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
 *  for more details.
 *
 *  You should have received a copy of the GNU General Public License along
 *  with this program; if not, write to the Free Software Foundation, Inc.,
 *  59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 *
 *  For any other enquiries write to Gary J. Olsen, Department of Microbiology,
 *  University of Illinois, Urbana, IL  61801, USA
 *
 *  Or send E-mail to gary@phylo.life.uiuc.edu
 *
 *
 *  fastDNAml is based in part on the program dnaml by Joseph Felsenstein.
 *
 *  Copyright notice from dnaml:
 *
 *     version 3.3. (c) Copyright 1986, 1990 by the University of Washington
 *     and Joseph Felsenstein.  Written by Joseph Felsenstein.  Permission is
 *     granted to copy and use this program provided no fee is charged for it
 *     and provided that this copyright notice is not removed.
 *
 *
 *  When publishing work that based on results from fastDNAml please cite:
 *
 *  Felsenstein, J.  1981.  Evolutionary trees from DNA sequences:
 *  A maximum likelihood approach.  J. Mol. Evol. 17: 368-376.
 *
 *  and
 *
 *  Olsen, G. J., Matsuda, H., Hagstrom, R., and Overbeek, R.  1994.
 *  fastDNAml:  A tool for construction of phylogenetic trees of DNA
 *  sequences using maximum likelihood.  Comput. Appl. Biosci. 10: 41-48.
 */

/*  Conversion to C and changes in sequential code by Gary Olsen, 1991-1994
 *
 *  p4 version by Hideo Matsuda and Ross Overbeek, 1991-1993
 */

/*
 *  1.0    March 14, 1992
 *         Initial "release" version
 *
 *  1.0.1  March 18, 1992
 *         Add ntaxa to tree comments
 *         Set minimum branch length on reading tree
 *         Add blanks around operators in treeString (for prolog parsing)
 *         Add program version to treeString comments
 *
 *  1.0.2  April 6, 1992
 *         Improved option line diagnostics
 *         Improved auxiliary line diagnostics
 *         Removed some trailing blanks from output
 *
 *  1.0.3  April 6, 1992
 *         Checkpoint trees that do not need any optimization
 *         Print restart tree likelihood before optimizing
 *         Fix treefile option so that it really toggles
 *
 *  1.0.4  July 13, 1992
 *         Add support for tree fact (instead of true Newick tree) in
 *            processTreeCom, treeReadLen, str_processTreeCom and
 *            str_treeReadLen
 *         Use bit operations in randum
 *         Correct error in bootstrap mask used with weighting mask
 *
 *  1.0.5  August 22, 1992
 *         Fix reading of underscore as first nonblank character in name
 *         Add strchr and strstr functions to source code
 *         Add output treefile name to message "Tree also written ..."
 *
 *  1.0.6  November 20, 1992
 *         Change (! nsites) test in setupTopol to (nsites == 0) for MIPS R4000
 *         Add vectorizing compiler directives for CRAY
 *         Include updates and corrections to parallel code from H. Matsuda
 *
 *  1.0.7  March 25, 1993
 *         Remove translation of underlines in taxon names
 *
 *  1.0.8  April 30, 1993
 *         Remove version number from fastDNAml.h file name
 *
 *  1.0.9  August 12, 1993
 *         Version was never released.
 *         Redefine treefile formats and default:
 *             0  None
 *             1  Newick
 *             2  Prolog
 *             3  PHYLIP (Default)
 *         Remove quote marks and comment from PHYLIP treefile format.
 *
 *  1.1.0  September 3-5, 1993
 *         Arrays of size maxpatterns moved from stack to heap (mallocs) in
 *            evaluateslope, makenewz, and cmpBestTrees.
 *         Correct [maxsites] to [maxpatterns] in temporary array definitions
 *            in Vectorize code of newview and evaluate.  (These should also
 *            get converted to use malloc() at some point.)
 *         Change randum to use 12 bit segments, not 6.  Change its seed
 *            parameter to long *.
 *         Remove the code that took the absolute value of random seeds.
 *         Correct integer divide artifact in setting default transition/
 *            transversion parameter values.
 *         When transition/transversion ratio is "reset", change to small
 *            value, not the program default.
 *         Report the "reset" transition/transversion ratio in the output.
 *         Move global data into analdef, rawDNA, and crunchedDNA structures.
 *         Change names of routines white and digit to whitechar and digitchar.
 *         Convert y[] to yType, which is normally char, but is int if the
 *            Vectorize flag is set.
 *         Split option line reading out of getoptions routine.
 *
 *  1.1.1  September 30, 1994
 *         Incorporate changes made in 1.0.A (Feb. 11, 1994):
 *            Remove processing of quotation marks within comments.
 *            Break label finding into copy to string and find tip.
 *            Generalize tree reading to read trees when names are and are not
 *               already known.
 *            Remove absolute value from randum seed reading.
 *         Include integer version number and program date.
 *         Remove maxsite, maxpatterns and maxsp limitations.
 *         Incorporate code for retaining multiple trees.
 *         Activate code for Hasegawa & Kishino test of tree differences.
 *         Make quick add the default, with Q turning it off.
 *         Make emperical frequencies the option with F turning it off.
 *         Allow a residue frequency option line anywhere in the options.
 *         Increase string length passed to treeString (should be length
 *               checked, but ...)
 *         Introduce (Sept.30) and fix (Oct. 26) bug in bootstrap sampling.
 *         Fix error when user frequencies are last line and start with F.
 *
 *  1.2    September 5, 1997
 *         Move likelihood components into structure.
 *         Change rawDNA to rawdata.
 *         Change crunchedDNA to cruncheddata.
 *         Recast the likelihoods per site into an array of stuctures,
 *               where each stucture (likelivector) includes the likelihoods
 *               of each residue type at the site, and a magnitude scale
 *               factor (exp).  This requires changing the space allocation,
 *               newview, makenewz, evaluate, and sigma.
 *         Change code of newview to rescale likelihoods up by 2**256 when
 *               the largest value falls below 2**-256.  This should solve
 *               floating point underflow for all practical sized trees.
 *               No changes are necessary in makenewz or sigma, since only
 *               relative likelihoods are necessary.
 *
 *  1.2.1  March 9, 1998
 *         Convert likelihood adjustment factor (2**256) to a constant.
 *         Fix vectorized calculation of likelihood (error introduced in 1.2)
 *
 *  1.2.2  December 23, 1998
 *         General code clean-up.
 *         Convert to function definitions with parameter type lists
 *
 *  1.2.2  January 3, 2000
 *         Add copyright and license information
 *         Make this the current release version
 */

#include <math.h>
#include <sys/times.h>
#include <sys/types.h>
#include <sys/time.h>
#include <time.h> 
#include <stdlib.h>
#include <stdio.h>
#include <string.h>
#include "axml.h"

#define RANDOM_TREE 1
#define RANDOM_RESULT 2
#define TREE_EVALUATE 3
#define TREE_RESULT 4
#define WORKER_READY 5
#define FINISH_AXML 6
#define TREE_OPTIMIZE 7
#define TREE_OPTIMIZE_RESULT 8
#define TREE_EVALUATE_VAR 9
#define TREE_RESULT_VAR 10
#define EXHAUSTIVE_TREE 11
#define EXHAUSTIVE_RESULT 12
#define TREE_CONSENSUS 13
#define TREE_CONSENSUS_RESULT 14
#define CTREES_ARRAY 15
#define FILE_NAME 16


#define programName        "AxMl"
#define programVersion     "1.1.0"
#define programVersionInt   10202
#define programDate        "February, 2002"
#define programDateInt      20000212



/*  Global variables */

xarray       *usedxtip, *freextip;
FILE  *INFILE;
/*AxMl global Variables */

int MAX_REARRANGE = 21;
int stepwidth = 5;
struct timeval ttime;
int auxLines = 0;
double globTime = 0.0;
char *myTreestr, *bestTreeStr;
double bestTreeLh;
int treeStringLength;

boolean ship_seq_data = FALSE;
extern char *optarg;                  
extern int  optind,opterr,optopt;      
int Multiple = 1;
int Thorough = 0;
char *seq_data_str = NULL, run_id[40] = "", workdir[2048] = "", seq_file[2048] = "", tree_file[2048]="";
permutationList *plist = NULL;
treelist *tl;
int topologyCounter;

permutationList *permutationListInsert(double val, char *treestr, permutationList *plist, tree *tr);
char *treeString (char *treestr, tree *tr, nodeptr p, int form);
char *getList(permutationList *plist, int k);
void insertList(tree *tr);
void resetList();

double gettime()
{
  gettimeofday(&ttime , NULL);
  return ttime.tv_sec + ttime.tv_usec * 0.000001;
}


/*=======================================================================*/
/*                              PROGRAM                                  */
/*=======================================================================*/
/*                    Best tree handling for dnaml                       */
/*=======================================================================*/

/*  Tip value comparisons
 *
 *  Use void pointers to hide type from other routines.  Only tipValPtr and
 *  cmpTipVal need to be changed to alter the nature of the values compared
 *  (e.g., names instead of node numbers).
 *
 *    cmpTipVal(tipValPtr(nodeptr p), tipValPtr(nodeptr q)) == -1, 0 or 1.
 *
 *  This provides direct comparison of tip values (for example, for
 *  definition of tr->start).
 */


void  *tipValPtr (nodeptr p)
  { return  (void *) & p->number;
    }


int  cmpTipVal (void *v1, void *v2)
  { /* cmpTipVal */
    int  i1, i2;

    i1 = *((int *) v1);
    i2 = *((int *) v2);
    return  (i1 < i2) ? -1 : ((i1 == i2) ? 0 : 1);
  } /* cmpTipVal */


/*  These are the only routines that need to UNDERSTAND topologies */











double str_readTreeLikelihood (char *treestr);











/*  Compare two trees, assuming that each is in standard order.  Return
 *  -1 if first preceeds second, 0 if they are identical, or +1 if first
 *  follows second in standard order.  Lower number tips preceed higher
 *  number tips.  A tip preceeds a corresponding internal node.  Internal
 *  nodes are ranked by their lowest number tip.
 */








/*=======================================================================*/
/*                       End of best tree routines                       */
/*=======================================================================*/




boolean getnums (rawdata *rdta)
    /* input number of species, number of sites */
  { /* getnums */
    

    if (fscanf(INFILE, "%d %d", & rdta->numsp, & rdta->sites) != 2) {
      printf("ERROR: Problem reading number of species and sites\n");
      return FALSE;
      }
   

    if (rdta->numsp < 4) {
      printf("TOO FEW SPECIES\n");
      return FALSE;
      }

    if (rdta->sites < 1) {
      printf("TOO FEW SITES\n");
      return FALSE;
      }

    return TRUE;
  } /* getnums */


boolean digitchar (int ch) {return (ch >= '0' && ch <= '9'); }


boolean whitechar (int ch)
   { return (ch == ' ' || ch == '\n' || ch == '\t');
     }


void uppercase (int *chptr)
    /* convert character to upper case -- either ASCII or EBCDIC */
  { /* uppercase */
    int  ch;

    ch = *chptr;
    if ((ch >= 'a' && ch <= 'i') || (ch >= 'j' && ch <= 'r')
                                 || (ch >= 's' && ch <= 'z'))
      *chptr = ch + 'A' - 'a';
  } /* uppercase */


int base36 (int ch)
  { /* base36 */
    if      (ch >= '0' && ch <= '9') return (ch - '0');
    else if (ch >= 'A' && ch <= 'I') return (ch - 'A' + 10);
    else if (ch >= 'J' && ch <= 'R') return (ch - 'J' + 19);
    else if (ch >= 'S' && ch <= 'Z') return (ch - 'S' + 28);
    else if (ch >= 'a' && ch <= 'i') return (ch - 'a' + 10);
    else if (ch >= 'j' && ch <= 'r') return (ch - 'j' + 19);
    else if (ch >= 's' && ch <= 'z') return (ch - 's' + 28);
    else return -1;
  } /* base36 */


int itobase36 (int i)
  { /* itobase36 */
    if      (i <  0) return '?';
    else if (i < 10) return (i      + '0');
    else if (i < 19) return (i - 10 + 'A');
    else if (i < 28) return (i - 19 + 'J');
    else if (i < 36) return (i - 28 + 'S');
    else return '?';
  } /* itobase36 */


int findch (int c)
  { /* findch */
    int ch;

    while ((ch = getc(INFILE)) != EOF && ch != c) ;
    return  ch;
  } /* findch */





boolean inputboot(analdef *adef)
    /* read the bootstrap auxilliary info */
  { /* inputboot */
    if (! adef->boot) {
      printf("ERROR: Unexpected Bootstrap auxiliary data line\n");
      return FALSE;
      }
    else if (fscanf(INFILE, "%ld", & adef->boot) != 1 ||
             findch('\n') == EOF) {
      printf("ERROR: Problem reading boostrap random seed value\n");
      return FALSE;
      }

    return TRUE;
  } /* inputboot */


boolean inputcategories (rawdata *rdta)
    /* read the category rates and the categories for each site */
  { /* inputcategories */
    int  i, j, ch, ci;

    if (rdta->categs >= 0) {
      printf("ERROR: Unexpected Categories auxiliary data line\n");
      return FALSE;
      }
    if (fscanf(INFILE, "%d", & rdta->categs) != 1) {
      printf("ERROR: Problem reading number of rate categories\n");
      return FALSE;
      }
    if (rdta->categs < 1 || rdta->categs > maxcategories) {
      printf("ERROR: Bad number of categories: %d\n", rdta->categs);
      printf("Must be in range 1 - %d\n", maxcategories);
      return FALSE;
      }

    for (j = 1; j <= rdta->categs
           && fscanf(INFILE, "%lf", &(rdta->catrat[j])) == 1; j++) ;

    if ((j <= rdta->categs) || (findch('\n') == EOF)) {
      printf("ERROR: Problem reading rate values\n");
      return FALSE;
      }

    for (i = 1; i <= nmlngth; i++)  (void) getc(INFILE);

    i = 1;
    while (i <= rdta->sites) {
      ch = getc(INFILE);
      ci = base36(ch);
      if (ci >= 0 && ci <= rdta->categs)
        rdta->sitecat[i++] = ci;
      else if (! whitechar(ch)) {
        printf("ERROR: Bad category character (%c) at site %d\n", ch, i);
        return FALSE;
        }
      }

    if (findch('\n') == EOF) {      /* skip to end of line */
      printf("ERROR: Missing newline at end of category data\n");
      return FALSE;
      }

    return TRUE;
  } /* inputcategories */


boolean inputextra (analdef *adef)
  { /* inputextra */
    if (fscanf(INFILE,"%d", & adef->extra) != 1 ||
        findch('\n') == EOF) {
      printf("ERROR: Problem reading extra info value\n");
      return FALSE;
      }

    return TRUE;
  } /* inputextra */


boolean inputfreqs (rawdata *rdta)
  { /* inputfreqs */
    if (fscanf(INFILE, "%lf%lf%lf%lf",
               & rdta->freqa, & rdta->freqc,
               & rdta->freqg, & rdta->freqt) != 4 ||
        findch('\n') == EOF) {
      printf("ERROR: Problem reading user base frequencies data\n");
      return  FALSE;
      }

    rdta->freqread = TRUE;
    return TRUE;
  } /* inputfreqs */


boolean inputglobal (tree *tr)
    /* input the global option information */
  { /* inputglobal */
    int  ch;

    if (tr->global != -2) {
      printf("ERROR: Unexpected Global auxiliary data line\n");
      return FALSE;
      }
    if (fscanf(INFILE, "%d", &(tr->global)) != 1) {
      printf("ERROR: Problem reading rearrangement region size\n");
      return FALSE;
      }
    if (tr->global < 0) {
      printf("WARNING: Global region size too small;\n");
      printf("         value reset to local\n\n");
      tr->global = 1;
      }
    else if (tr->global == 0)  tr->partswap = 0;
    else if (tr->global > tr->mxtips - 3) {
      tr->global = tr->mxtips - 3;   
      }

    while ((ch = getc(INFILE)) != '\n') {  /* Scan for second value */
      if (! whitechar(ch)) {
        if (ch != EOF)  (void) ungetc(ch, INFILE);
        if (ch == EOF || fscanf(INFILE, "%d", &(tr->partswap)) != 1
                      || findch('\n') == EOF) {
          printf("ERROR: Problem reading insert swap region size\n");
          return FALSE;
          }
        else if (tr->partswap < 0)  tr->partswap = 1;
        else if (tr->partswap > tr->mxtips - 3) {
          tr->partswap = tr->mxtips - 3;
          }

        if (tr->partswap > tr->global)  tr->global = tr->partswap;
        break;   /*  Break while loop */
        }
      }

    return TRUE;
  } /* inputglobal */


boolean inputjumble (analdef *adef)
  { /* inputjumble */
    if (! adef->jumble) {
      printf("ERROR: Unexpected Jumble auxiliary data line\n");
      return FALSE;
      }
    else if (fscanf(INFILE, "%ld", & adef->jumble) != 1 ||
             findch('\n') == EOF) {
      printf("ERROR: Problem reading jumble random seed value\n");
      return FALSE;
      }
    else if (adef->jumble == 0) {
      printf("WARNING: Jumble random number seed is zero\n\n");
      }

    return TRUE;
  } /* inputjumble */


boolean inputkeep (analdef *adef)
  { /* inputkeep */
    if (fscanf(INFILE, "%d", & adef->nkeep) != 1 ||
        findch('\n') == EOF || adef->nkeep < 1) {
      printf("ERROR: Problem reading number of kept trees\n");
      return FALSE;
      }

    return TRUE;
  } /* inputkeep */


boolean inputoutgroup (analdef *adef, tree *tr)
  { /* inputoutgroup */
    if (! adef->root || tr->outgr > 0) {
      printf("ERROR: Unexpected Outgroup auxiliary data line\n");
      return FALSE;
      }
    else if (fscanf(INFILE, "%d", &(tr->outgr)) != 1 ||
             findch('\n') == EOF) {
      printf("ERROR: Problem reading outgroup number\n");
      return FALSE;
      }
    else if ((tr->outgr < 1) || (tr->outgr > tr->mxtips)) {
      printf("ERROR: Bad outgroup: '%d'\n", tr->outgr);
      return FALSE;
      }

    return TRUE;
  } /* inputoutgroup */


boolean inputratio (rawdata *rdta)
  { /* inputratio */
    if (rdta->ttratio >= 0.0) {
      printf("ERROR: Unexpected Transition/transversion auxiliary data\n");
      return FALSE;
      }
    else if (fscanf(INFILE,"%lf", & rdta->ttratio)!=1 ||
             findch('\n') == EOF) {
      printf("ERROR: Problem reading transition/transversion ratio\n");
      return FALSE;
      }

    return TRUE;
  } /* inputratio */


/*  Y 0 is treeNone   (no tree)
    Y 1 is treeNewick
    Y 2 is treeProlog
    Y 3 is treePHYLIP
 */

boolean inputtreeopt (analdef *adef)
  { /* inputtreeopt */
    if (! adef->trout) {
      printf("ERROR: Unexpected Treefile auxiliary data\n");
      return FALSE;
      }
    else if (fscanf(INFILE,"%d", & adef->trout) != 1 ||
             findch('\n') == EOF) {
      printf("ERROR: Problem reading output tree-type number\n");
      return FALSE;
      }
    else if ((adef->trout < 0) || (adef->trout > treeMaxType)) {
      printf("ERROR: Bad output tree-type number: '%d'\n", adef->trout);
      return FALSE;
      }

    return TRUE;
  } /* inputtreeopt */


boolean inputweights (analdef *adef, rawdata *rdta, cruncheddata *cdta)
    /* input the character weights 0, 1, 2 ... 9, A, B, ... Y, Z */
  { /* inputweights */
    int i, ch, wi;

    if (! adef->userwgt || cdta->wgtsum > 0) {
      printf("ERROR: Unexpected Weights auxiliary data\n");
      return FALSE;
      }

    for (i = 2; i <= nmlngth; i++)  (void) getc(INFILE);
    cdta->wgtsum = 0;
    i = 1;
    while (i <= rdta->sites) {
      ch = getc(INFILE);
      wi = base36(ch);
      if (wi >= 0)
        cdta->wgtsum += rdta->wgt[i++] = wi;
      else if (! whitechar(ch)) {
        printf("ERROR: Bad weight character: '%c'", ch);
        printf("       Weights in dnaml must be a digit or a letter.\n");
        return FALSE;
        }
      }

    if (findch('\n') == EOF) {      /* skip to end of line */
      printf("ERROR: Missing newline at end of weight data\n");
      return FALSE;
      }

    return TRUE;
  } /* inputweights */




boolean getoptions (analdef *adef, rawdata *rdta, cruncheddata *cdta, tree *tr)
  { /* getoptions */
    int     ch, i, extranum;

    adef->boot    =           0;  /* Don't bootstrap column weights */
    adef->empf    =        TRUE;  /* Use empirical base frequencies */
    adef->extra   =           0;  /* No extra runtime info unless requested */
    adef->interleaved =    TRUE;  /* By default, data format is interleaved */
    adef->jumble  =       FALSE;  /* Use random addition sequence */
    adef->nkeep   =           0;  /* Keep only the one best tree */
    adef->prdata  =       FALSE;  /* Don't echo data to output stream */
    adef->qadd    =        TRUE;  /* Smooth branches globally in add */
    adef->root    =       FALSE;  /* User-defined outgroup rooting */
    adef->trout   = treeDefType;  /* Output tree file */
    adef->trprint =        TRUE;  /* Print tree to output stream */
    rdta->categs  =           0;  /* No rate categories */
    rdta->catrat[1] =       1.0;  /* Rate values */
    rdta->freqread =      FALSE;  /* User-defined frequencies not read yet */
    rdta->ttratio =         2.0;  /* Transition/transversion rate ratio */
    tr->global    =          -1;  /* Default search locale for optimum */
    tr->mxtips    = rdta->numsp;
    tr->outgr     =           1;  /* Outgroup number */
    tr->partswap  =           1;  /* Default to swap locally after insert */
    tr->userlen   =       FALSE;  /* User-supplied branch lengths */
    adef->usertree =      FALSE;  /* User-defined tree topologies */
    adef->userwgt =       FALSE;  /* User-defined position weights */
    extranum      =           0;

    while ((ch = getc(INFILE)) != '\n' && ch != EOF) {
      uppercase(& ch);
      switch (ch) {
          case '1' : adef->prdata  = ! adef->prdata; break;
          case '3' : adef->trprint = ! adef->trprint; break;
          case '4' : adef->trout   = treeDefType - adef->trout; break;
          case 'B' : adef->boot    =  1; extranum++; break;
          case 'C' : rdta->categs  = -1; extranum++; break;
          case 'E' : adef->extra   = -1; break;
          case 'F' : adef->empf    = ! adef->empf; break;
          case 'G' : tr->global    = -2; break;
          case 'I' : adef->interleaved = ! adef->interleaved; break;
          case 'J' : adef->jumble  = 1; extranum++; break;
          case 'K' : extranum++; break;
          case 'L' : tr->userlen   = TRUE; break;
          case 'O' : adef->root    = TRUE; tr->outgr = 0; extranum++; break;
          case 'Q' : adef->qadd    = FALSE; break;
          case 'T' : rdta->ttratio = -1.0; extranum++; break;
          case 'U' : adef->usertree = TRUE; break;
          case 'W' : adef->userwgt = TRUE; cdta->wgtsum = 0; extranum++; break;
          case 'Y' : adef->trout   = treeDefType - adef->trout; break;
          case ' ' : break;
          case '\t': break;
          default  :
              printf("ERROR: Bad option character: '%c'\n", ch);
              return FALSE;
          }
      }

    if (ch == EOF) {
      printf("ERROR: End-of-file in options list\n");
      return FALSE;
      }

    if (adef->usertree && adef->restart) {
      printf("ERROR:  The restart and user-tree options conflict:\n");
      printf("        Restart adds rest of taxa to a starting tree;\n");
      printf("        User-tree does not add any taxa.\n\n");
      return FALSE;
      }

    if (adef->usertree && adef->jumble) {
      printf("WARNING:  The jumble and user-tree options conflict:\n");
      printf("          Jumble adds taxa to a tree in random order;\n");
      printf("          User-tree does not use taxa addition.\n");
      printf("          Jumble option cancelled for this run.\n\n");
      adef->jumble = FALSE;
      }

    if (tr->userlen && tr->global != -1) {
      printf("ERROR:  The global and user-lengths options conflict:\n");
      printf("        Global optimizes a starting tree;\n");
      printf("        User-lengths constrain the starting tree.\n\n");
      return FALSE;
      }

    if (tr->userlen && ! adef->usertree) {
      printf("WARNING:  User lengths required user tree option.\n");
      printf("          User-tree option set for this run.\n\n");
      adef->usertree = TRUE;
      }

    rdta->wgt      = (int *)    malloc((rdta->sites + 1) * sizeof(int));
    rdta->wgt2     = (int *)    malloc((rdta->sites + 1) * sizeof(int));
    rdta->sitecat  = (int *)    malloc((rdta->sites + 1) * sizeof(int));
    cdta->alias    = (int *)    malloc((rdta->sites + 1) * sizeof(int));
    cdta->aliaswgt = (int *)    malloc((rdta->sites + 1) * sizeof(int));
    cdta->patcat   = (int *)    malloc((rdta->sites + 1) * sizeof(int));
    cdta->patrat   = (double *) malloc((rdta->sites + 1) * sizeof(double));
    cdta->wr       = (double *) malloc((rdta->sites + 1) * sizeof(double));
    cdta->wr2      = (double *) malloc((rdta->sites + 1) * sizeof(double));
    if ( ! rdta->wgt || ! rdta->wgt2     || ! rdta->sitecat || ! cdta->alias
                     || ! cdta->aliaswgt || ! cdta->patcat  || ! cdta->patrat
                     || ! cdta->wr       || ! cdta->wr2) {
      fprintf(stderr, "getoptions: Malloc failure\n");
      return 0;
      }

    /*  process lines with auxiliary data */

    auxLines = extranum;
    if(!adef->empf) auxLines++;

    while (extranum--) {
      ch = getc(INFILE);
      uppercase(& ch);
      switch (ch) {
        case 'B':  if (! inputboot(adef)) return FALSE; break;
        case 'C':  if (! inputcategories(rdta)) return FALSE; break;
        case 'E':  if (! inputextra(adef)) return FALSE; extranum++; break;
        case 'F':  if (! inputfreqs(rdta)) return FALSE; break;
        case 'G':  if (! inputglobal(tr)) return FALSE; extranum++; break;	      	
        case 'J':  if (! inputjumble(adef)) return FALSE; break;
        case 'K':  if (! inputkeep(adef)) return FALSE; break;
        case 'O':  if (! inputoutgroup(adef, tr)) return FALSE; break;
        case 'T':  if (! inputratio(rdta)) return FALSE; break;
        case 'W':  if (! inputweights(adef, rdta, cdta)) return FALSE; break;
        case 'Y':  if (! inputtreeopt(adef)) return FALSE; extranum++; break;

        default:
            printf("ERROR: Auxiliary options line starts with '%c'\n", ch);
            return FALSE;
        }
      }

    if (! adef->userwgt) {
      for (i = 1; i <= rdta->sites; i++) rdta->wgt[i] = 1;
      cdta->wgtsum = rdta->sites;
      }

    if (adef->userwgt && cdta->wgtsum < 1) {
      printf("ERROR:  Missing or bad user-supplied weight data.\n");
      return FALSE;
      }

    if (adef->boot) {
      printf("Bootstrap random number seed = %ld\n\n", adef->boot);
      }

    if (adef->jumble) {
      printf("Jumble random number seed = %ld\n\n", adef->jumble);
      }

    if (adef->qadd) {
      
      }

    if (rdta->categs > 0) {
     
      for (i = 1; i <= rdta->sites; i++) {
        if ((rdta->wgt[i] > 0) && (rdta->sitecat[i] < 1)) {
          printf("ERROR: Bad category (%c) at site %d\n",
                  itobase36(rdta->sitecat[i]), i);
          return FALSE;
          }
        }
      }
    else if (rdta->categs < 0) {
      printf("ERROR: Category auxiliary data missing from input\n");
      return FALSE;
      }
    else {                                        /* rdta->categs == 0 */
      for (i = 1; i <= rdta->sites; i++) rdta->sitecat[i] = 1;
      rdta->categs = 1;
      }

    if (tr->outgr < 1) {
      printf("ERROR: Outgroup auxiliary data missing from input\n");
      return FALSE;
      }

    if (rdta->ttratio < 0.0) {
      printf("ERROR: Transition/transversion auxiliary data missing from input\n");
      return FALSE;
      }

    if (tr->global < 0) {
      if (tr->global == -2) tr->global = tr->mxtips - 3;  /* Default global */
      else                  tr->global = adef->usertree ? 0 : 1;/* No global */
      }


    if (adef->usertree && ! tr->global) {
      printf("User-supplied tree topology%swill be used.\n\n",
        tr->userlen ? " and branch lengths " : " ");
      }
    else {
      if (! adef->usertree) {
        
        }
      
      }

    if (! adef->usertree && adef->nkeep == 0) adef->nkeep = 1;

    

    return TRUE;
  } /* getoptions */


boolean getbasefreqs (rawdata *rdta)
  { /* getbasefreqs */
    int  ch;

    if (rdta->freqread) return TRUE;

    ch = getc(INFILE);
    if (! ((ch == 'F') || (ch == 'f')))  (void) ungetc(ch, INFILE);

    if (fscanf(INFILE, "%lf%lf%lf%lf",
               & rdta->freqa, & rdta->freqc,
               & rdta->freqg, & rdta->freqt) != 4 ||
        findch('\n') == EOF) {
      printf("ERROR: Problem reading user base frequencies\n");
      return  FALSE;
      }

    return TRUE;
  } /* getbasefreqs */


boolean getyspace (rawdata *rdta)
  { /* getyspace */
    long   size;
    int    i;
    yType *y0;

    if (! (rdta->y = (yType **) malloc((rdta->numsp + 1) * sizeof(yType *)))) {
      printf("ERROR: Unable to obtain space for data array pointers\n");
      return  FALSE;
      }

    size = 4 * (rdta->sites / 4 + 1);
    if (! (y0 = (yType *) malloc((rdta->numsp + 1) * size * sizeof(yType)))) {
      printf("ERROR: Unable to obtain space for data array\n");
      return  FALSE;
      }

    rdta->y0 = y0;

    for (i = 0; i <= rdta->numsp; i++) {
      rdta->y[i] = y0;
      y0 += size;
      }

    return  TRUE;
  } /* getyspace */


void freeyspace(rawdata *rdta)
{
  long   size;
  int    i;
  yType *y0;
  for (i = 0; i <= rdta->numsp; i++) 
    {
      free(rdta->y[i]);    
    }
}



boolean setupTree (tree *tr, int nsites)
  { /* setupTree */
    nodeptr  p0, p, q;
    int      i, j, tips, inter;

    tips  = tr->mxtips;
    inter = tr->mxtips - 1;

    if (!(p0 = (nodeptr) malloc((tips + 3*inter) * sizeof(node)))) {
      printf("ERROR: Unable to obtain sufficient tree memory\n");
      return  FALSE;
      }


    if (!(tr->nodep = (nodeptr *) malloc((2*tr->mxtips) * sizeof(nodeptr)))) {
      printf("ERROR: Unable to obtain sufficient tree memory, too\n");
      return  FALSE;
      }

    
    tr->nodep[0] = (node *) NULL;    /* Use as 1-based array */

    for (i = 1; i <= tips; i++) {    /* Set-up tips */
      p = p0++;
      p->x      = (xarray *) NULL;
      p->tip    = (yType *) NULL;
      
      /* AxML modification start*/
#ifdef SEV_CODE
      p->equalityVector = (char *) NULL; /*initialize eq vector with zero */
#endif      
      /* AxML modification end*/


      p->number = i;
      p->next   = p;
      p->back   = (node *) NULL;

      tr->nodep[i] = p;
      }

    for (i = tips + 1; i <= tips + inter; i++) { /* Internal nodes */
      q = (node *) NULL;
      for (j = 1; j <= 3; j++) {
        p = p0++;
        p->x      = (xarray *) NULL;
        p->tip    = (yType *) NULL;
        p->number = i;

	/* AxML modification start*/
#ifdef SEV_CODE
	p->equalityVector = (char *) NULL; /*initialize eq vector with zero */	
#endif
	/* AxML modification end*/


        p->next   = q;
        p->back   = (node *) NULL;
        q = p;
        }
      p->next->next->next = p;
      tr->nodep[i] = p;
      }

    tr->likelihood  = unlikely;
    tr->start       = (node *) NULL;
    tr->outgrnode   = tr->nodep[tr->outgr];
    tr->ntips       = 0;
    tr->nextnode    = 0;
    tr->opt_level   = 0;
    tr->prelabeled  = TRUE;
    tr->smoothed    = FALSE;


    

    return TRUE;
  } /* setupTree */

#ifdef VECTOR_CODE
void freeTreeNode (nodeptr p)   /* Free tree node (sector) associated data */
  { /* freeTreeNode */
    if (p) {
      if (p->x) {
        if (p->x->a) free(p->x->a);
	if (p->x->c) free(p->x->c);
	if (p->x->g) free(p->x->g);
	if (p->x->t) free(p->x->t);
	if (p->x->exp) free(p->x->exp);
        free(p->x);
        }
   
      }
  } /* freeTree */
#endif
#ifndef VECTOR_CODE
void freeTreeNode (nodeptr p)   /* Free tree node (sector) associated data */
  { /* freeTreeNode */
    if (p) {
      if (p->x) {
        if (p->x->lv) free(p->x->lv);
        free(p->x);
        }
      /* AxML modification start*/
 #ifdef SEV_CODE
      free(p->equalityVector); /*free equality vector of node*/
#endif      
      /* AxML modification end*/
      }
  } /* freeTree */
#endif


/* AxML modification start*/

boolean setupEqualityVector(tree *tr) /* new function for initializing and mapping equality vectors */
{
  int siteNum = tr->cdta->endsite;
  nodeptr upper = tr->nodep[tr->mxtips * 2 - 1];
  nodeptr p = tr->nodep[1];
  char *ev, *end;
  char *tip;   
  likelivector *l;
  int i;
  homType *mlvP;
  int ref[EQUALITIES];
  int code;
 
 
  


  
  for(; p < upper; p++)
    {      
      if( ! (p->equalityVector = 
	     (ev = (char *)malloc(siteNum * sizeof(char)))))
	{
	  printf("ERROR: Failure to get node equality vector  memory\n");
	  return  FALSE; 
	}      
      
      


      end = & ev[siteNum];
      
      if(p->tip)
	{	 
	  int i;
	  tip = p->tip;
	  mlvP = p->mlv;
	  
	  for(i = 0; i < equalities; i++)
	    ref[i] = -1;

	  for(i = 0; i < siteNum; i++)
	    {	    
	      ev[i] = map[tip[i]];  	  	     	      	     	      
	      
	      if(ref[ev[i]] == -1)
		ref[ev[i]] = 1;
	    }
	  for(i = 0; i < equalities; i++)
	    {	      
	      code = map2[i];
	      if(ref[i] == 1)
		{
		  mlvP->a =  code       & 1;
		  mlvP->c = (code >> 1) & 1; 	
		  mlvP->g = (code >> 2) & 1;
		  mlvP->t = (code >> 3) & 1;
		  mlvP->exp = 0;
		  mlvP->set = TRUE;
		}
	      else
		mlvP->set = FALSE;

	      mlvP++;	 
	    }
	}
      else
	{	 	
	  for(; ev < end; ev++)
	    {
	      *ev = 0;	     
	    }
	}
    } 
  
  
  /*freeyspace(tr->rdta);*/

  return TRUE;
}


  /* AxML modification end*/







void freeTree (tree *tr)
  { /* freeTree */
    nodeptr  p, q;
    int  i, tips, inter;

    tips  = tr->mxtips;
    inter = tr->mxtips - 1;

    for (i = 1; i <= tips; i++) freeTreeNode(tr->nodep[i]);

    for (i = tips + 1; i <= tips + inter; i++) {
      if (p = tr->nodep[i]) {
        if (q = p->next) {
          freeTreeNode(q->next);
          freeTreeNode(q);
          }
        freeTreeNode(p);
        }
      }
    

    free(tr->nodep[1]);       /* Free the actual nodes */
  } /* freeTree */


boolean getdata (analdef *adef, rawdata *rdta, tree *tr)
    /* read sequences */
  { /* getdata */
    int   i, j, k, l, basesread, basesnew, ch;
    int   meaning[256];          /*  meaning of input characters */ 
    char *nameptr;
    boolean  allread, firstpass;

    for (i = 0; i <= 255; i++) meaning[i] = 0;
    meaning['A'] =  1;
    meaning['B'] = 14;
    meaning['C'] =  2;
    meaning['D'] = 13;
    meaning['G'] =  4;
    meaning['H'] = 11;
    meaning['K'] = 12;
    meaning['M'] =  3;
    meaning['N'] = 15;
    meaning['O'] = 16;
    meaning['R'] =  5;
    meaning['S'] =  6;
    meaning['T'] =  8;
    meaning['U'] =  8;
    meaning['V'] =  7;
    meaning['W'] =  9;
    meaning['X'] = 17;
    meaning['Y'] = 10;
    meaning['?'] = 18;
    meaning['-'] = 19;

    basesread = basesnew = 0;

    allread = FALSE;
    firstpass = TRUE;
    ch = ' ';

    while (! allread) {
      for (i = 1; i <= tr->mxtips; i++) {     /*  Read data line */

        if (firstpass) {                      /*  Read species names */
          j = 1;
          while (whitechar(ch = getc(INFILE))) {  /*  Skip blank lines */
            if (ch == '\n')  j = 1;  else  j++;
            }

          if (j > nmlngth) {
            printf("ERROR: Blank name for species %d; ", i);
            printf("check number of species,\n");
            printf("       number of sites, and interleave option.\n");
            return  FALSE;
            }

          nameptr = tr->nodep[i]->name;
          for (k = 1; k < j; k++)  *nameptr++ = ' ';

          while (ch != '\n' && ch != EOF) {
            if (whitechar(ch))  ch = ' ';
            *nameptr++ = ch;
            if (++j > nmlngth) break;
            ch = getc(INFILE);
            }

          while (*(--nameptr) == ' ') ;          /*  remove trailing blanks */
          *(++nameptr) = '\0';                   /*  add null termination */

          if (ch == EOF) {
            printf("ERROR: End-of-file in name of species %d\n", i);
            return  FALSE;
            }
          }    /* if (firstpass) */



        j = basesread;
        while ((j < rdta->sites)
               && ((ch = getc(INFILE)) != EOF)
               && ((! adef->interleaved) || (ch != '\n'))) {
          uppercase(& ch);
          if (meaning[ch] || ch == '.') {
            j++;
            if (ch == '.') {
              if (i != 1) ch = rdta->y[1][j];
              else {
                printf("ERROR: Dot (.) found at site %d of sequence 1\n", j);
                return  FALSE;
                }
              }
            rdta->y[i][j] = ch;
            }
          else if (whitechar(ch) || digitchar(ch)) ;
          else {
            printf("ERROR: Bad base (%c) at site %d of sequence %d\n",
                    ch, j, i);
            return  FALSE;
            }
          }

        if (ch == EOF) {
          printf("ERROR: End-of-file at site %d of sequence %d\n", j, i);
          return  FALSE;
          }

        if (! firstpass && (j == basesread)) i--;        /* no data on line */
        else if (i == 1) basesnew = j;
        else if (j != basesnew) {
          printf("ERROR: Sequences out of alignment\n");
          printf("%d (instead of %d) residues read in sequence %d %s\n",
		 j - basesread, basesnew - basesread, i, tr->nodep[i]->name);
          return  FALSE;
          }

        while (ch != '\n' && ch != EOF) ch = getc(INFILE);  /* flush line */
        }                                                  /* next sequence */
      firstpass = FALSE;
      basesread = basesnew;
      allread = (basesread >= rdta->sites);
      }

    /*  Print listing of sequence alignment */

    if (adef->prdata) {
      j = nmlngth - 5 + ((rdta->sites + ((rdta->sites - 1)/10))/2);
      if (j < nmlngth - 1) j = nmlngth - 1;
      if (j > 37) j = 37;
      printf("Name"); for (i=1;i<=j;i++) putchar(' '); printf("Sequences\n");
      printf("----"); for (i=1;i<=j;i++) putchar(' '); printf("---------\n");
      putchar('\n');

      for (i = 1; i <= rdta->sites; i += 60) {
        l = i + 59;
        if (l > rdta->sites) l = rdta->sites;

        if (adef->userwgt) {
          printf("Weights   ");
          for (j = 11; j <= nmlngth+3; j++) putchar(' ');
          for (k = i; k <= l; k++) {
            putchar(itobase36(rdta->wgt[k]));
            if (((k % 10) == 0) && (k < l)) putchar(' ');
            }
          putchar('\n');
          }

        if (rdta->categs > 1) {
          printf("Categories");
          for (j = 11; j <= nmlngth+3; j++) putchar(' ');
          for (k = i; k <= l; k++) {
            putchar(itobase36(rdta->sitecat[k]));
            if (((k % 10) == 0) && (k < l)) putchar(' ');
            }
          putchar('\n');
          }

        for (j = 1; j <= tr->mxtips; j++) {
          nameptr = tr->nodep[j]->name;
          k = nmlngth+3;
          while (ch = *nameptr++) {putchar(ch); k--;}
          while (--k >= 0) putchar(' ');

          for (k = i; k <= l; k++) {
            ch = rdta->y[j][k];
            if ((j > 1) && (ch == rdta->y[1][k])) ch = '.';
            putchar(ch);
            if (((k % 10) == 0) && (k < l)) putchar(' ');
            }
          putchar('\n');
          }
        putchar('\n');
        }
      }

    for (j = 1; j <= tr->mxtips; j++)    /* Convert characters to meanings */
      for (i = 1; i <= rdta->sites; i++) {
        rdta->y[j][i] = meaning[rdta->y[j][i]];
        }

    return  TRUE;
  } /* getdata */


boolean  getntrees (analdef *adef)
  { /* getntrees */

    if (fscanf(INFILE, "%d", &(adef->numutrees)) != 1 || findch('\n') == EOF) {
      printf("ERROR: Problem reading number of user trees\n");
      return  FALSE;
      }

    if (adef->nkeep == 0) adef->nkeep = adef->numutrees;

    return  TRUE;
  } /* getntrees */

FILE *mb; 
char bayesFileName[1024];

void modifyName(char *in,  char *out)
{
  int i = 0;
  do 
    {
      if(in[i] == '-') out[i] = '_';
      else out[i] = in[i];
      i++;
    }
  while(in[i] != '\0');
  out[i] = '\0'; 
}

boolean getinput (analdef *adef, rawdata *rdta, cruncheddata *cdta, tree *tr)
  { /* getinput */
    int i,j;
    char inverseMap[21] = {'z', 'A', 'C','M','G','R','S','V', 'T', 'W', 'Y','H','K','D','B','N','O','X','?','-'};
    char modifiedName[nmlngth];
    char **names;
    if (! getnums(rdta))                       return FALSE;
    if (! getoptions(adef, rdta, cdta, tr))    return FALSE;
   
    if (! adef->empf && ! getbasefreqs(rdta))  return FALSE;
    if (! getyspace(rdta))                     return FALSE;
    if (! setupTree(tr, rdta->sites))          return FALSE;
    if (! getdata(adef, rdta, tr))             return FALSE;
    names = (char **)malloc((tr->mxtips + 1) * sizeof(char *));
    strcpy(bayesFileName, workdir);
    strcat(bayesFileName,"//");
    strcat(bayesFileName, run_id);
    strcat(bayesFileName, ".mb");
    mb = fopen(bayesFileName, "w");
   
    fprintf(mb, "#NEXUS\n\n");
    fprintf(mb, "BEGIN DATA;\n");
    fprintf(mb, "DIMENSIONS NTAX=%d NCHAR=%d;\n", tr->mxtips, rdta->sites);    
    fprintf(mb, "FORMAT DATATYPE=DNA INTERLEAVE MISSING=-;\n");
    for(i = 1; i <= tr->mxtips; i++)
      names[i] = tr->nodep[i]->name;
    for(i = 1; i <= tr->mxtips; i++)
    {                 	  	
      modifyName(tr->nodep[i]->name, modifiedName);
      fprintf(mb, "[Name: %s  Len: %d Check: 0]\n",modifiedName, rdta->sites);
    }

    RAxML_parsimony_init(tr->mxtips, rdta->sites, rdta->y, names);

    fprintf(mb,"\n \n MATRIX \n");
    for(i = 1; i <= tr->mxtips; i++)
      {
	modifyName(tr->nodep[i]->name, modifiedName);
	fprintf(mb, "%s ", modifiedName);
	
	for(j = 1; j <= rdta->sites; j++)
	  {	    	   
	    if(rdta->y[i][j] > 15) rdta->y[i][j] = 15; 
	    fprintf(mb, "%c", inverseMap[rdta->y[i][j]]);
	  }
	fprintf(mb,"\n");       
      }
    if (adef->usertree && ! getntrees(adef))   return FALSE;
    free(names);
    return TRUE;
  } /* getinput */





void sitesort (rawdata *rdta, cruncheddata *cdta)
    /* Shell sort keeping sites with identical residues and weights in
     * the original order (i.e., a stable sort).
     * The index created in cdta->alias is 1 based.  The
     * sitecombcrunch routine packs it to a 0 based index.
     */
  { /* sitesort */
    int  gap, i, j, jj, jg, k, n, nsp;
    int  *index, *category;
    boolean  flip, tied;
    yType  **data;

    index    = cdta->alias;
    category = rdta->sitecat;
    data     = rdta->y;
    n        = rdta->sites;
    nsp      = rdta->numsp;

    for (gap = n / 2; gap > 0; gap /= 2) {
      for (i = gap + 1; i <= n; i++) {
        j = i - gap;

        do {
          jj = index[j];
          jg = index[j+gap];
          flip = (category[jj] >  category[jg]);
          tied = (category[jj] == category[jg]);
          for (k = 1; (k <= nsp) && tied; k++) {
            flip = (data[k][jj] >  data[k][jg]);
            tied = (data[k][jj] == data[k][jg]);
            }
          if (flip) {
            index[j]     = jg;
            index[j+gap] = jj;
            j -= gap;
            }
          } while (flip && (j > 0));

        }  /* for (i ...   */
      }    /* for (gap ... */
  } /* sitesort */


void sitecombcrunch (rawdata *rdta, cruncheddata *cdta)
    /* combine sites that have identical patterns (and nonzero weight) */
  { /* sitecombcrunch */
    int  i, sitei, j, sitej, k;
    int gaps = 0;
    boolean  tied;

    /* AxML modif eliminate columns with lots of gaps */
    /* Pattern reduction method */

   

    /*AxMl modif */

    i = 0;
    cdta->alias[0] = cdta->alias[1];
    cdta->aliaswgt[0] = 0;

    for (j = 1; j <= rdta->sites; j++) {
      sitei = cdta->alias[i];
      sitej = cdta->alias[j];
      tied = (rdta->sitecat[sitei] == rdta->sitecat[sitej]);

      for (k = 1; tied && (k <= rdta->numsp); k++)
        tied = (rdta->y[k][sitei] == rdta->y[k][sitej]);

      if (tied) {
        cdta->aliaswgt[i] += rdta->wgt2[sitej];
        }
      else {
        if (cdta->aliaswgt[i] > 0) i++;
        cdta->aliaswgt[i] = rdta->wgt2[sitej];
        cdta->alias[i] = sitej;
        }
      }

    cdta->endsite = cdta->endsiteNormal = i;
    if (cdta->aliaswgt[i] > 0) cdta->endsite++;
  } /* sitecombcrunch */


boolean makeweights (analdef *adef, rawdata *rdta, cruncheddata *cdta)
    /* make up weights vector to avoid duplicate computations */
  { /* makeweights */
    int  i;

    
    for (i = 1; i <= rdta->sites; i++)  rdta->wgt2[i] = rdta->wgt[i];

    for (i = 1; i <= rdta->sites; i++)  cdta->alias[i] = i;
    sitesort(rdta, cdta);
    sitecombcrunch(rdta, cdta);
    return TRUE;
  } /* makeweights */






boolean makevalues (rawdata *rdta, cruncheddata *cdta)
    /* set up fractional likelihoods at tips */
  { /* makevalues */
    double  temp, wtemp;
    int  i, j;

    for (i = 1; i <= rdta->numsp; i++) {    /* Pack and move tip data */
      for (j = 0; j < cdta->endsite; j++) {
        rdta->y[i-1][j] = rdta->y[i][cdta->alias[j]];
        }
      }

    for (j = 0; j < cdta->endsite; j++) {
      cdta->patcat[j] = i = rdta->sitecat[cdta->alias[j]];
      cdta->patrat[j] = temp = rdta->catrat[i];
      cdta->wr[j]  = wtemp = temp * cdta->aliaswgt[j];
      cdta->wr2[j] = temp * wtemp;
      }

   

    return TRUE;
  } /* makevalues */


boolean empiricalfreqs (rawdata *rdta, cruncheddata *cdta)
    /* Get empirical base frequencies from the data */
  { /* empiricalfreqs */
    double  sum, suma, sumc, sumg, sumt, wj, fa, fc, fg, ft;
    int     i, j, k, code;
    yType  *yptr;

    rdta->freqa = 0.25;
    rdta->freqc = 0.25;
    rdta->freqg = 0.25;
    rdta->freqt = 0.25;
    for (k = 1; k <= 8; k++) {
      suma = 0.0;
      sumc = 0.0;
      sumg = 0.0;
      sumt = 0.0;
      for (i = 0; i < rdta->numsp; i++) {
        yptr = rdta->y[i];
        for (j = 0; j < cdta->endsite; j++) {
          code = *yptr++;
          fa = rdta->freqa * ( code       & 1);
          fc = rdta->freqc * ((code >> 1) & 1);
          fg = rdta->freqg * ((code >> 2) & 1);
          ft = rdta->freqt * ((code >> 3) & 1);
          wj = cdta->aliaswgt[j] / (fa + fc + fg + ft);
          suma += wj * fa;
          sumc += wj * fc;
          sumg += wj * fg;
          sumt += wj * ft;
          }
        }
      sum = suma + sumc + sumg + sumt;
      rdta->freqa = suma / sum;
      rdta->freqc = sumc / sum;
      rdta->freqg = sumg / sum;
      rdta->freqt = sumt / sum;
      }

    return TRUE;
  } /* empiricalfreqs */


void reportfreqs (analdef *adef, rawdata *rdta)
  { /* reportfreqs */
    double  suma, sumb;

   
    rdta->freqr = rdta->freqa + rdta->freqg;
    rdta->invfreqr = 1.0/rdta->freqr;
    rdta->freqar = rdta->freqa * rdta->invfreqr;
    rdta->freqgr = rdta->freqg * rdta->invfreqr;
    rdta->freqy = rdta->freqc + rdta->freqt;
    rdta->invfreqy = 1.0/rdta->freqy;
    rdta->freqcy = rdta->freqc * rdta->invfreqy;
    rdta->freqty = rdta->freqt * rdta->invfreqy;
    
    suma = rdta->ttratio*rdta->freqr*rdta->freqy
         - (rdta->freqa*rdta->freqg + rdta->freqc*rdta->freqt);
    sumb = rdta->freqa*rdta->freqgr + rdta->freqc*rdta->freqty;
    rdta->xi = suma/(suma+sumb);
    rdta->xv = 1.0 - rdta->xi;
    if (rdta->xi <= 0.0) {
      printf("WARNING: This transition/transversion ratio\n");
      printf("         is impossible with these base frequencies!\n");
      printf("Transition/transversion parameter reset\n\n");
      rdta->xi = 0.000001;
      rdta->xv = 1.0 - rdta->xi;
      rdta->ttratio = (sumb * rdta->xi / rdta->xv 
                       + rdta->freqa * rdta->freqg
                       + rdta->freqc * rdta->freqt)
                    / (rdta->freqr * rdta->freqy);
     
      }
   
    rdta->fracchange = 2.0 * rdta->xi * (rdta->freqa * rdta->freqgr
                                       + rdta->freqc * rdta->freqty)
                     + rdta->xv * (1.0 - rdta->freqa * rdta->freqa
                                       - rdta->freqc * rdta->freqc
                                       - rdta->freqg * rdta->freqg
                                       - rdta->freqt * rdta->freqt);
  } /* reportfreqs */


boolean linkdata2tree (rawdata *rdta, cruncheddata *cdta, tree *tr)
    /* Link data array to the tree tips */
  { /* linkdata2tree */
    int  i;
    int  j;
    yType *tip;
    int ref[EQUALITIES];
    tr->rdta       = rdta;
    tr->cdta       = cdta;

    for(i = 0; i < EQUALITIES; i++)
      ref[i] = 0;
    equalities = 0;


    for (i = 1; i <= tr->mxtips; i++) {    /* Associate data with tips */
      tr->nodep[i]->tip = &(rdta->y[i-1][0]);
      tip =  tr->nodep[i]->tip;
      for(j = 0; j < tr->cdta->endsite; j++)
	{
	  if(ref[tip[j]] == 0)
	    ref[tip[j]] = 1; 
	}
      }

     for(i = 0; i < EQUALITIES; i++)
       {
	 if(ref[i] == 1) 
	   {
	     map[i] = equalities++;	     	   
	   }
	 else
	   map[i] = -1;
       }
     
     map2 = (char *)malloc(equalities * sizeof(char));

     j = 0;

     for(i = 0; i < EQUALITIES; i++)
       {
	 if(ref[i] == 1)
	   {
	     map2[j] = i;
	     j++;
	   }
       }
  
    

    return TRUE;
  } /* linkdata2tree */

#ifdef VECTOR_CODE
xarray *setupxarray (int npat)
  { /* setupxarray */
    xarray        *x;
    xtype *a, *c, *g, *t;
    long *exp;

    /* grafikkarte */

    x = (xarray *) malloc(sizeof(xarray));
    if (x) {
      a = (xtype *) malloc(npat * sizeof(xtype));
      c = (xtype *) malloc(npat * sizeof(xtype));
      g = (xtype *) malloc(npat * sizeof(xtype));
      t = (xtype *) malloc(npat * sizeof(xtype));
      exp = (long *) malloc(npat * sizeof(long));
      if (a && c && g && t && exp) {
        x->a = a;
	x->c = c;
	x->g = g;
	x->t = t;
	x->exp = exp;
        x->prev = x->next = x;
        x->owner = (node *) NULL;
        }
      else {
        free(x);
        return (xarray *) NULL;
        }
      }
    return x;
  } /* setupxarray */
#endif
#ifndef VECTOR_CODE
xarray *setupxarray (int npat)
  { /* setupxarray */
    xarray        *x;
    likelivector  *data;

    x = (xarray *) malloc(sizeof(xarray));
    if (x) {
      data = (likelivector *) malloc(npat * sizeof(likelivector));
      if (data) {
        x->lv = data;
        x->prev = x->next = x;
        x->owner = (node *) NULL;
        }
      else {
        free(x);
        return (xarray *) NULL;
        }
      }
    return x;
  } /* setupxarray */
#endif

boolean linkxarray (int req, int min, int npat,
                    xarray **freexptr, xarray **usedxptr)
    /*  Link a set of xarrays */
  { /* linkxarray */
    xarray  *first, *prev, *x;
    int  i;

    first = prev = (xarray *) NULL;
    i = 0;

    do {
      x = setupxarray(npat);
      if (x) {
        if (! first) first = x;
        else {
          prev->next = x;
          x->prev = prev;
          }
        prev = x;
        i++;
        }
      else {
        printf("ERROR: Failure to get requested xarray memory\n");
        if (i < min)  return  FALSE;
        }
      } while ((i < req) && x);

    if (first) {
      first->prev = prev;
      prev->next = first;
      }

    *freexptr = first;
    *usedxptr = (xarray *) NULL;

    return  TRUE;
  } /* linkxarray */


boolean setupnodex (tree *tr)
  { /* setupnodex */
    nodeptr  p;
    int  i;

    for (i = tr->mxtips + 1; (i <= 2*(tr->mxtips) - 2); i++) {
      p = tr->nodep[i];
      if (! (p->x = setupxarray(tr->cdta->endsite))) {
        printf("ERROR: Failure to get internal node xarray memory\n");
        return  FALSE;
        }
      }

    return  TRUE;
  } /* setupnodex */


xarray *getxtip (nodeptr p)
  { /* getxtip */
    xarray  *new;
    boolean  splice;

    if (! p) return (xarray *) NULL;

    splice = FALSE;

    if (p->x) {                  /* array is there; move to tail of list */
      new = p->x;
      if (new == new->prev) ;             /* linked to self; leave it */
      else if (new == usedxtip) usedxtip = usedxtip->next; /* at head */
      else if (new == usedxtip->prev) ;   /* already at tail */
      else {                              /* move to tail of list */
        new->prev->next = new->next;
        new->next->prev = new->prev;
        splice = TRUE;
        }
      }

    else if (freextip) {                 /* take from unused list */
      p->x = new = freextip;
      new->owner = p;
      if (new->prev != new) {            /* not only member of freelist */
        new->prev->next = new->next;
        new->next->prev = new->prev;
        freextip = new->next;
        }
      else
        freextip = (xarray *) NULL;

      splice = TRUE;
      }

    else if (usedxtip) {                 /* take from head of used list */
      usedxtip->owner->x = (xarray *) NULL;
      p->x = new = usedxtip;
      new->owner = p;
      usedxtip = usedxtip->next;
      }

    else {
      printf("ERROR: Unable to locate memory for tip %d.\n", p->number);
      return  (xarray *) NULL;
      }

    if (splice) {
      if (usedxtip) {                  /* list is not empty */
        usedxtip->prev->next = new;
        new->prev = usedxtip->prev;
        usedxtip->prev = new;
        new->next = usedxtip;
        }
      else
        usedxtip = new->prev = new->next = new;
      }

    return  new;
  } /* getxtip */


xarray *getxnode (nodeptr p)
    /* Ensure that internal node p has memory */
  { /* getxnode */
    nodeptr  s;

    if (! (p->x)) {  /*  Move likelihood array on this node to sector p */
      if ((s = p->next)->x || (s = s->next)->x) {
        p->x = s->x;
        s->x = (xarray *) NULL;
        }
      else {
        printf("ERROR: Unable to locate memory at node %d.\n", p->number);
        exit(1);
        }
      }
    return  p->x;
  } /* getxnode */


 /* AxML modification start*/
 /* optimized version of function newview(), implementing homogeneous and */
 /* partially heterogeneous subtree equality vectors */

#ifdef VECTOR_CODE
void scalarMultAndAdd(double *result, double scalar1, double *vector1, 
		      double scalar2, double *vector2, int n)
{
  int i;

  for(i = 0; i < n; i++)
    result[i] = scalar1 * vector1[i] + scalar2 * vector2[i];
}

void addComponents(double *result, double *vector1, double *vector2, int n)
{
  int i;

  for(i = 0; i < n; i++)
    result[i] = vector1[i] + vector2[i];
}

void addComponentsLONG(long *result, long *vector1, long *vector2, int n)
{
  int i;

  for(i = 0; i < n; i++)
    result[i] = vector1[i] + vector2[i];
}

void scalarMult(double *result, double scalar, int n)
{
  int i;

  for(i = 0; i < n; i++)
    result[i] *= scalar;
}

void scalarMult2(double *result, double scalar, double *v, int n)
{
  int i;

  for(i = 0; i < n; i++)
    result[i] = v[i] * scalar;
}


void complex3Operands(double *result, double *vector1, double *vector2, 
		      double *vector3, int n)
{
  int i;

  for(i = 0; i < n; i++)
    result[i] = vector1[i] * (vector2[i] - vector3[i]) + vector3[i];
}

void complex4Operands(double *result, double *vector1, double *vector2, 
		      double *vector3, double *vector4, int n)
{
  int i;

  for(i = 0; i < n; i++)
    result[i] = vector1[i] * (vector2[i] - vector3[i]) + vector4[i];
}

void veryComplexOp(double *result, double *vector1, double *vector2, 
		   double *vector3, double *vector4, double *vector5, int n)
{
  int i;

  for(i = 0; i < n; i++)
    result[i] = vector1[i] * (vector2[i] * (vector3[i] - vector4[i]) + vector5[i]);
}

void scaleVector(double *vector1, double *vector2, double *vector3, 
		 double *vector4, long *vector5, int n)
{
  int i;
  
  
  for(i = 0; i < n; i++)
    {
      if (vector1[i] < minlikelihood && vector2[i] < minlikelihood
	  && vector3[i] < minlikelihood && vector4[i] < minlikelihood) 
	{
	  vector1[i]   *= twotothe256;
	  vector2[i]   *= twotothe256;
	  vector3[i]   *= twotothe256;
	  vector4[i]   *= twotothe256;
	  vector5[i] += 1;
	} 
    }

}

void add2Components(double *result, double *vector1, int n)
{
  int i;

  for(i = 0; i < n; i++)
    result[i] += vector1[i];
}
void substract2Components(double *result, double *vector1, int n)
{
  int i;

  for(i = 0; i < n; i++)
    result[i] -= vector1[i];
}

void mnzComplex(double *result, double *v1, double *v2,double *v3, double *v4,
		double *v5, double *v6,double *v7, double *v8, int n)
{
  int i;
  for(i = 0; i < n; i++)
    result[i] = v1[i] * v2[i] + v3[i] * v4[i] + v5[i] * v6[i] + v7[i] * v8[i];
    
}

void mnzMultAndAdd(double *result, double *v1, double *v2,double s3, double *v4,
		double *v5, double s6, int n)
{
  int i;
  for(i = 0; i < n; i++)
    result[i] = v1[i] * v2[i] * s3 + v4[i] * v5[i] * s6;
    
}

void mnzMultAdd(double *result, double *v1, double *v2, double *v3,
		double *v4, int n)
{
  int i;
  for(i = 0; i < n; i++)
    result[i] = (v1[i] + v2[i]) * (v3[i] + v4[i]);
    
}

void evaluateComplex(double *result, double *v1, double *v2, double *v3, double *v4, double *v5, 
		     long *i1, long *i2, int n)
{
 int i;
 for(i = 0; i < n; i++)
   result[i] = log(v1[i] * v2[i] + v3[i] * v4[i] + v5[i]) + 
     (i1[i] + i2[i])*log(minlikelihood);
}


void sumEvaluate(double *sum, int *i1, double *v1, int n)
{
  int i;
  for(i = 0; i < n; i++)
    *sum += i1[i] * v1[i];

}
void multComponents(double *result, double *v1, int n)
{
  int i;
  for(i = 0; i < n; i++)
    result[i] *= v1[i];

}
void add3Vectors(double *result, double *v1, double *v2, double *v3, int n)
{
  int i;
  for(i = 0; i < n; i++)
    result[i] = v1[i] + v2[i] + v3[i];

}

void makenewzConditional(double *result, double *v1, double *v2, double *v3, double *v4, int n)
{
  int i;
  for(i = 0; i < n; i++)
    result[i] = ((v1[i] + v2[i]) == 0.0) ? 1.0 : v3[i] / v4[i];
}
void makenewzConditional2(double *result, double *v1, double *v2, double s, double *v3, 
			  double *v4, int n)
{
  int i;
  for(i = 0; i < n; i++)
    result[i] = ((v1[i] + v2[i]) == 0.0) ? s : s * v3[i] / v4[i];
}


void sumMakenewz1(double *result, double *v1, double *v2, int n)
{
  int i;
  for(i = 0; i < n; i++)
    *result += v1[i]  * v2[i];

} 

void sumMakenewz2(double *result, double *v1, double *v2, double s, double *v3, 
		  double *v4,  int n)
{
  int i;
  for(i = 0; i < n; i++)
    *result += v1[i] * (v2[i] + s * v3[i] - v4[i] * v4[i]);

} 





boolean newview (tree *tr, nodeptr p)        /*  Update likelihoods at node */
  { /* newview */
    double   zq, lzq, xvlzq, zr, lzr, xvlzr;
    nodeptr  q, r;
    xtype *lpa, *lqa, *lra, *lpc, *lqc, *lrc, 
      *lpt, *lqt, *lrt,*lpg, *lqg, *lrg;
    long *pexp, *qexp, *rexp; 
    int  i;
    double t;

    if (p->tip) {             /*  Make sure that data are at tip */
      int           code;
      yType        *yptr;

      if (p->x) return TRUE;  /*  They are already there */

      if (! getxtip(p)) return FALSE; /*  They are not, so get memory */
      lpa = p->x->a;
      lpg = p->x->g;
      lpt = p->x->t;
      lpc = p->x->c;
      pexp = p->x->exp;
      yptr = p->tip;          /*  Pointer to first nucleotide datum */
      for (i = 0; i < tr->cdta->endsite; i++) {
        code = yptr[i];
        lpa[i] =  code       & 1;
        lpc[i] = (code >> 1) & 1;
        lpg[i] = (code >> 2) & 1;
        lpt[i] = (code >> 3) & 1;
        pexp[i] = 0;        
        }
      return TRUE;
      }

/*  Internal node needs update */

    q = p->next->back;
    r = p->next->next->back;

    while ((! p->x) || (! q->x) || (! r->x)) {
      if (! q->x) if (! newview(tr, q))  return FALSE;
      if (! r->x) if (! newview(tr, r))  return FALSE;
      if (! p->x) if (! getxnode(p)) return FALSE;
      }

    lpa = p->x->a;
    lpc = p->x->c;
    lpg = p->x->g;
    lpt = p->x->t;
    pexp = p->x->exp;

    lqa = q->x->a;
    lqc = q->x->c;
    lqg = q->x->g;
    lqt = q->x->t;
    qexp = q->x->exp;

    zq = q->z;
    lzq = (zq > zmin) ? log(zq) : log(zmin);
    xvlzq = tr->rdta->xv * lzq;

    lra = r->x->a;
    lrc = r->x->c;
    lrg = r->x->g;
    lrt = r->x->t;
    rexp = r->x->exp;

    zr = r->z;
    lzr = (zr > zmin) ? log(zr) : log(zmin);
    xvlzr = tr->rdta->xv * lzr;

    { double  zzqtable[maxcategories+1], zvqtable[maxcategories+1],
              zzrtable[maxcategories+1], zvrtable[maxcategories+1],
             *zzqptr, *zvqptr, *zzrptr, *zvrptr, *rptr;
      double  *fxqr, *fxqy, *fxqn, *sumaq, *sumgq, *sumcq, *sumtq, ki,
	*tempj;
      int  *cptr;


      double *zzq, *zvq,*zzr, *zvr;
      int  cat;
      zzq = (double *)malloc(tr->cdta->endsite * sizeof(double));
      zvq = (double *)malloc(tr->cdta->endsite * sizeof(double));
      zzr = (double *)malloc(tr->cdta->endsite * sizeof(double));
      zvr = (double *)malloc(tr->cdta->endsite * sizeof(double));
      fxqr = (double *)malloc(tr->cdta->endsite * sizeof(double));
      fxqy = (double *)malloc(tr->cdta->endsite * sizeof(double));
      fxqn = (double *)malloc(tr->cdta->endsite * sizeof(double));
      sumaq = (double *)malloc(tr->cdta->endsite * sizeof(double));
      sumgq = (double *)malloc(tr->cdta->endsite * sizeof(double));
      sumcq = (double *)malloc(tr->cdta->endsite * sizeof(double));
      sumtq = (double *)malloc(tr->cdta->endsite * sizeof(double));
      tempj = (double *)malloc(tr->cdta->endsite * sizeof(double));	

       


      rptr   = &(tr->rdta->catrat[1]);
      zzqptr = &(zzqtable[1]);
      zvqptr = &(zvqtable[1]);
      zzrptr = &(zzrtable[1]);
      zvrptr = &(zvrtable[1]);


      for (i = 1; i <= tr->rdta->categs; i++) {   /* exps for each category */
        ki = *rptr++;
        *zzqptr++ = exp(ki *   lzq);
        *zvqptr++ = exp(ki * xvlzq);
        *zzrptr++ = exp(ki *   lzr);
        *zvrptr++ = exp(ki * xvlzr);
        }

      cptr = &(tr->cdta->patcat[0]);


      for (i = 0; i < tr->cdta->endsite; i++) 
	{
          cat = *cptr++;
          zzq[i] = zzqtable[cat];
          zvq[i] = zvqtable[cat];
          zzr[i] = zzrtable[cat];
          zvr[i] = zvrtable[cat];
        }

     

      scalarMultAndAdd(fxqr,  tr->rdta->freqa, lqa, tr->rdta->freqg, lqg, tr->cdta->endsite);
      scalarMultAndAdd(fxqy,  tr->rdta->freqc, lqc, tr->rdta->freqt, lqt, tr->cdta->endsite);
      addComponents(fxqn, fxqr, fxqy, tr->cdta->endsite);
      scalarMult(fxqr, tr->rdta->invfreqr, tr->cdta->endsite); 
      scalarMult(fxqy, tr->rdta->invfreqy, tr->cdta->endsite);
      complex3Operands(tempj,  zvq, fxqr, fxqn, tr->cdta->endsite);
      complex4Operands(sumaq, zzq, lqa, fxqr, tempj,tr->cdta->endsite);
      complex4Operands(sumgq, zzq, lqg, fxqr, tempj,tr->cdta->endsite);
      complex3Operands(tempj, zvq, fxqy, fxqn, tr->cdta->endsite);
      complex4Operands(sumcq, zzq, lqc, fxqy, tempj, tr->cdta->endsite);
      complex4Operands(sumtq, zzq, lqt, fxqy, tempj, tr->cdta->endsite);
      scalarMultAndAdd(fxqr, tr->rdta->freqa, lra, tr->rdta->freqg, lrg, tr->cdta->endsite);
      scalarMultAndAdd(fxqy, tr->rdta->freqc, lrc, tr->rdta->freqt, lrt, tr->cdta->endsite);
      addComponents(fxqn, fxqr, fxqy, tr->cdta->endsite);
      scalarMult(fxqr, tr->rdta->invfreqr, tr->cdta->endsite);
      scalarMult(fxqy, tr->rdta->invfreqy, tr->cdta->endsite);
      complex3Operands(tempj, zvr, fxqr, fxqn, tr->cdta->endsite);
      veryComplexOp(lpa, sumaq, zzr, lra, fxqr, tempj, tr->cdta->endsite);
      veryComplexOp(lpg, sumgq, zzr, lrg, fxqr, tempj, tr->cdta->endsite);
      complex3Operands(tempj, zvr, fxqy, fxqn, tr->cdta->endsite);
      veryComplexOp(lpc, sumcq, zzr, lrc, fxqy, tempj, tr->cdta->endsite);
      veryComplexOp(lpt, sumtq, zzr, lrt, fxqy, tempj, tr->cdta->endsite);
      addComponentsLONG( pexp, qexp, rexp, tr->cdta->endsite);
      scaleVector(lpa, lpg, lpc, lpt, pexp, tr->cdta->endsite);
      
     

      /*for (i = 0; i < tr->cdta->endsite; i++) {
	          fxqr[i] = tr->rdta->freqa * lqa[i] + tr->rdta->freqg * lqg[i];
          fxqy[i] = tr->rdta->freqc * lqc[i] + tr->rdta->freqt * lqt[i];	  
	
          fxqn[i] = fxqr[i] + fxqy[i];
	  fxqr[i] *= tr->rdta->invfreqr;
	  fxqy[i] *= tr->rdta->invfreqy;
          
	  tempj[i] = zvq[i] * (fxqr[i] - fxqn[i]) + fxqn[i];
          sumaq[i] = zzq[i] * (lqa[i] - fxqr[i]) + tempj[i];
          sumgq[i] = zzq[i] * (lqg[i] - fxqr[i]) + tempj[i];

          tempj[i] = zvq[i] * (fxqy[i] - fxqn[i]) + fxqn[i];
          sumcq[i] = zzq[i] * (lqc[i] - fxqy[i]) + tempj[i];
          sumtq[i] = zzq[i] * (lqt[i] - fxqy[i]) + tempj[i];



          fxqr[i] = tr->rdta->freqa * lra[i] + tr->rdta->freqg * lrg[i];
          fxqy[i] = tr->rdta->freqc * lrc[i] + tr->rdta->freqt * lrt[i];
          fxqn[i] = fxqr[i] + fxqy[i];
	  
          fxqr[i] *= tr->rdta->invfreqr;
	  fxqy[i] *= tr->rdta->invfreqy;

	  tempj[i] = zvr[i] * (fxqr[i] - fxqn[i]) + fxqn[i];
          lpa[i] = sumaq[i] * (zzr[i] * (lra[i] - fxqr[i]) + tempj[i]);
          lpg[i] = sumgq[i] * (zzr[i] * (lrg[i] - fxqr[i]) + tempj[i]);
          
          tempj[i] = zvr[i] * (fxqy[i]-fxqn[i]) + fxqn[i];
          lpc[i] = sumcq[i] * (zzr[i] * (lrc[i] - fxqy[i]) + tempj[i]);
          lpt[i] = sumtq[i] * (zzr[i] * (lrt[i] - fxqy[i]) + tempj[i]);

          pexp[i] = qexp[i] + rexp[i];


          if (lpa[i] < minlikelihood && lpg[i] < minlikelihood
           && lpc[i] < minlikelihood && lpt[i] < minlikelihood) {
            lpa[i]   *= twotothe256;
            lpg[i]   *= twotothe256;
            lpc[i]   *= twotothe256;
            lpt[i]   *= twotothe256;
            pexp[i] += 1;
            }
      }
      */
      free(zvr);
      free(zvq);
      free(zzq);
      free(zzr);
      free(fxqr);
      free(fxqy);
      free(fxqn);
      free(tempj);
      free(sumaq);
      free(sumgq);
      free(sumcq);
      free(sumtq);

      return TRUE;
      }
  } /* newview */



double evaluate (tree *tr, nodeptr p)
{ /* evaluate */
    double   sum, z, lz, xvlz,
      ki, *fxpa, *fxpc, *fxpg, *fxpt,
             *suma, *sumc;
    double   *zz, *zv;
    double        zztable[maxcategories+1], zvtable[maxcategories+1],
                 *zzptr, *zvptr;
    double        *rptr;
    xtype *lpa, *lpc, *lpg, *lpt, *lqa, *lqc, *lqt, *lqg;
    long *pexp, *qexp;
    nodeptr       q;
    int           cat, *cptr, i, *wptr;
    double t;

    zz = (double *)malloc(tr->cdta->endsite * sizeof(double));
    zv = (double *)malloc(tr->cdta->endsite * sizeof(double));
    fxpa = (double *)malloc(tr->cdta->endsite * sizeof(double));
    fxpg = (double *)malloc(tr->cdta->endsite * sizeof(double));
    fxpc = (double *)malloc(tr->cdta->endsite * sizeof(double));
    fxpt = (double *)malloc(tr->cdta->endsite * sizeof(double));
    suma = (double *)malloc(tr->cdta->endsite * sizeof(double));
    sumc = (double *)malloc(tr->cdta->endsite * sizeof(double));
      


    q = p->back;
    while ((! p->x) || (! q->x)) {
      if (! (p->x)) if (! newview(tr, p)) return badEval;
      if (! (q->x)) if (! newview(tr, q)) return badEval;
      }

    lpa = p->x->a;
    lpc = p->x->c;
    lpg = p->x->g;
    lpt = p->x->t;
    pexp = p->x->exp;

    lqa = q->x->a;
    lqc = q->x->c;
    lqg = q->x->g;
    lqt = q->x->t;
    qexp = q->x->exp;

    z = p->z;
    if (z < zmin) z = zmin;
    lz = log(z);
    xvlz = tr->rdta->xv * lz;

    rptr  = &(tr->rdta->catrat[1]);
    zzptr = &(zztable[1]);
    zvptr = &(zvtable[1]);
    wptr = &(tr->cdta->aliaswgt[0]);
    cptr = &(tr->cdta->patcat[0]);
   
   
    sum = 0.0;
    for (i = 1; i <= tr->rdta->categs; i++) 
      {
	ki = *rptr++;
	*zzptr++ = exp(ki *   lz);
	*zvptr++ = exp(ki * xvlz);
      }

    for (i = 0; i < tr->cdta->endsite; i++) 
      {
        cat   = *cptr++;
        zz[i] = zztable[cat];
        zv[i] = zvtable[cat];
      }

    

      scalarMult2(fxpa, tr->rdta->freqa, lpa, tr->cdta->endsite);
      scalarMult2(fxpg, tr->rdta->freqg, lpg, tr->cdta->endsite);
      scalarMult2(fxpc, tr->rdta->freqc, lpc, tr->cdta->endsite);
      scalarMult2(fxpt, tr->rdta->freqt, lpt, tr->cdta->endsite);      
      mnzComplex(suma, fxpa, lqa, fxpc, lqc, fxpg, lqg, fxpt, lqt, tr->cdta->endsite);
      add2Components(fxpa, fxpg, tr->cdta->endsite);
      add2Components(fxpc, fxpt, tr->cdta->endsite);
      scalarMultAndAdd(fxpg, tr->rdta->freqa, lqa, tr->rdta->freqg, lqg, tr->cdta->endsite);
      scalarMultAndAdd(fxpt, tr->rdta->freqc, lqc, tr->rdta->freqt, lqt, tr->cdta->endsite);
      
      mnzMultAdd(sumc, fxpa, fxpc, fxpg, fxpt, tr->cdta->endsite);
      mnzMultAndAdd(fxpa, fxpa, fxpg, tr->rdta->invfreqr, fxpc, fxpt, 
		 tr->rdta->invfreqy, tr->cdta->endsite);
      substract2Components(suma, fxpa, tr->cdta->endsite);
      substract2Components(fxpa, sumc, tr->cdta->endsite);
      
      evaluateComplex(fxpg, zz, suma, zv, fxpa, sumc, pexp, qexp, tr->cdta->endsite);
      sumEvaluate(&sum, wptr, fxpg, tr->cdta->endsite);
      
    

      
      /*for (i = 0; i < tr->cdta->endsite; i++) {
        fxpa[i] = tr->rdta->freqa * lpa[i];
        fxpg[i] = tr->rdta->freqg * lpg[i];
        fxpc[i] = tr->rdta->freqc * lpc[i];
        fxpt[i] = tr->rdta->freqt * lpt[i];
	
        suma[i] = fxpa[i] * lqa[i] + fxpc[i] * lqc[i] + fxpg[i] * lqg[i] + fxpt[i] * lqt[i];
	
	fxpa[i] += fxpg[i];
        fxpc[i] += fxpt[i];
       
        fxpg[i] = tr->rdta->freqa * lqa[i] + tr->rdta->freqg * lqg[i];
        fxpt[i] = tr->rdta->freqc * lqc[i] + tr->rdta->freqt * lqt[i];
        
        
	sumc[i] = (fxpa[i] + fxpc[i]) * (fxpg[i] + fxpt[i]);
	
        fxpa[i] = fxpa[i] * fxpg[i] * tr->rdta->invfreqr + fxpc[i] * fxpt[i] * tr->rdta->invfreqy;
        suma[i] -= fxpa[i];
        fxpa[i] -= sumc[i];
	
        fxpg[i] = log(zz[i] * suma[i] + zv[i] * fxpa[i] + sumc[i]) + 
	(pexp[i] + qexp[i])*log(minlikelihood);
        sum += wptr[i] * fxpg[i];        
        }*/
      free(zz);
      free(zv);
      free(fxpa);
      free(fxpg);
      free(fxpc);
      free(fxpt);
      free(suma);
      free(sumc);
    tr->likelihood = sum;
    return  sum;
  } /* evaluate */





double makenewz (tree *tr, nodeptr p, nodeptr q, double z0, int maxiter)
  { /* makenewz */
     xtype *lpa, *lpc, *lpg, *lpt, *lqa, *lqc, *lqt, *lqg;
    long *pexp, *qexp;
    double  *abi, *bci, *sumci, *abptr, *bcptr, *sumcptr;
    double   *dlnLidlz, dlnLdlz, d2lnLdlz2, z, zprev, zstep, lz, xvlz,
      ki, *suma, *sumb, sumc, ab, bc, inv_Li, *t1, *t2,
             *fx1a, *fx1c, *fx1g, *fx1t, *fx2r, *fx2y, *zz, *zv;
    double   zztable[maxcategories+1], zvtable[maxcategories+1],
            *zzptr, *zvptr;
    double  *rptr, *wrptr, *wr2ptr;
    int      cat, *cptr, i, curvatOK;
    double t;

    while ((! p->x) || (! q->x)) {
      if (! (p->x)) if (! newview(tr, p)) return badZ;
      if (! (q->x)) if (! newview(tr, q)) return badZ;
      }
    lpa = p->x->a;
    lpc = p->x->c;
    lpg = p->x->g;
    lpt = p->x->t;
    pexp = p->x->exp;

    lqa = q->x->a;
    lqc = q->x->c;
    lqg = q->x->g;
    lqt = q->x->t;
    qexp = q->x->exp;
    

    dlnLidlz = fx1a = (double *)malloc(tr->cdta->endsite * sizeof(double));;
    t1  = fx1g = (double *)malloc(tr->cdta->endsite * sizeof(double));;
    t2 = fx1c = (double *)malloc(tr->cdta->endsite * sizeof(double));;
    zz = fx1t = (double *)malloc(tr->cdta->endsite * sizeof(double));;
    zv = (double *)malloc(tr->cdta->endsite * sizeof(double));;
    


    { unsigned scratch_size;
      scratch_size = sizeof(double) * tr->cdta->endsite;
      if ((abi   = (double *) malloc(scratch_size)) &&
          (bci   = (double *) malloc(scratch_size)) &&
          (sumci = (double *) malloc(scratch_size))) ;
      else {
        printf("ERROR: makenewz unable to obtain space for arrays\n");
        return badZ;
        }
      }
    abptr   = abi;
    bcptr   = bci;
    sumcptr = sumci;

   

    scalarMult2(fx1a, tr->rdta->freqa,lpa, tr->cdta->endsite);
    scalarMult2(fx1c, tr->rdta->freqc,lpc, tr->cdta->endsite);
    scalarMult2(fx1g, tr->rdta->freqg,lpg, tr->cdta->endsite);
    scalarMult2(fx1t, tr->rdta->freqt,lpt, tr->cdta->endsite);
    mnzComplex(abptr, fx1a, lqa, fx1c, lqc, fx1g, lqg, fx1t, lqt, tr->cdta->endsite);
    add2Components(fx1a, fx1g, tr->cdta->endsite);
    add2Components(fx1c, fx1t, tr->cdta->endsite);
    scalarMultAndAdd(fx1g, tr->rdta->freqa, lqa, tr->rdta->freqg, lqg, tr->cdta->endsite); 
    scalarMultAndAdd(fx1t, tr->rdta->freqc, lqc, tr->rdta->freqt, lqt, tr->cdta->endsite);
    mnzMultAndAdd(bcptr, fx1a, fx1g, tr->rdta->invfreqr, fx1c, 
		  fx1t, tr->rdta->invfreqy, tr->cdta->endsite);
    mnzMultAdd(sumcptr, fx1a, fx1c, fx1g, fx1t, tr->cdta->endsite);
    substract2Components(abptr, bcptr, tr->cdta->endsite);
    substract2Components(bcptr, sumcptr, tr->cdta->endsite);

   

    /*
    for (i = 0; i < tr->cdta->endsite; i++) {
      fx1a[i] = tr->rdta->freqa * lpa[i];
      fx1g[i] = tr->rdta->freqg * lpg[i];
      fx1c[i] = tr->rdta->freqc * lpc[i];
      fx1t[i] = tr->rdta->freqt * lpt[i];
      abptr[i] = fx1a[i] * lqa[i] + fx1c[i] * lqc[i] + fx1g[i] * lqg[i] + fx1t[i] * lqt[i];
      
      fx1a[i] = fx1a[i] + fx1g[i];
      fx1c[i] = fx1c[i] + fx1t[i];
      
      fx1g[i] = tr->rdta->freqa * lqa[i] + tr->rdta->freqg * lqg[i];
      fx1t[i] = tr->rdta->freqc * lqc[i] + tr->rdta->freqt * lqt[i];

      bcptr[i] = fx1a[i] * fx1g[i] * tr->rdta->invfreqr + 
	fx1c[i] * fx1t[i] * tr->rdta->invfreqy;
      
      sumcptr[i] = (fx1a[i] + fx1c[i]) * (fx1g[i] + fx1t[i]);      
      abptr[i]   = abptr[i] - bcptr[i];
      bcptr[i]   = bcptr[i] - sumcptr[i];
      }
    */
    z = z0;
    do {
      zprev = z;
      zstep = (1.0 - zmax) * z + zmin;
      curvatOK = FALSE;

      do {
        if (z < zmin) z = zmin;
        else if (z > zmax) z = zmax;

        lz    = log(z);
        xvlz  = tr->rdta->xv * lz;
        rptr  = &(tr->rdta->catrat[1]);
        zzptr = &(zztable[1]);
        zvptr = &(zvtable[1]);



        for (i = 1; i <= tr->rdta->categs; i++) {
          ki = *rptr++;
          *zzptr++ = exp(ki *   lz);
          *zvptr++ = exp(ki * xvlz);
          }
	cptr    = &(tr->cdta->patcat[0]);
	for (i = 0; i < tr->cdta->endsite; i++) 
	  {
	    cat   = *cptr++;
	    zz[i] = zztable[cat];
	    zv[i] = zvtable[cat];
	  }
	
	


        abptr   = abi;
        bcptr   = bci;
        sumcptr = sumci;
       
        wrptr   = &(tr->cdta->wr[0]);
        wr2ptr  = &(tr->cdta->wr2[0]);
        dlnLdlz = 0.0;                 /*  = d(ln(likelihood))/d(lz) */
        d2lnLdlz2 = 0.0;               /*  = d2(ln(likelihood))/d(lz)2 */




	multComponents(zz, abptr, tr->cdta->endsite);
	multComponents(zv, bcptr, tr->cdta->endsite);
	add3Vectors(dlnLidlz, zz, zv, sumcptr, tr->cdta->endsite);
	makenewzConditional(t1, zv, sumcptr, zz, dlnLidlz, tr->cdta->endsite);
	makenewzConditional2(t2, zz, sumcptr, tr->rdta->xv, zv, dlnLidlz, tr->cdta->endsite);
	addComponents(dlnLidlz, t1, t2, tr->cdta->endsite);
	sumMakenewz1(&dlnLdlz, wrptr, dlnLidlz, tr->cdta->endsite);
	sumMakenewz2(&d2lnLdlz2, wr2ptr, t1, tr->rdta->xv, t2, dlnLidlz, tr->cdta->endsite);
	for (i = 0; i < tr->cdta->endsite; i++) {
          /*zz[i]     = abptr[i] * zz[i];
	    zv[i]     = bcptr[i] * zv[i];*/
         
	  /* diese op geht nur mit double */
          /*inv_Li = 1.0/(ab + bc + sumc);
          t1     = zz[i] * inv_Li;
          t2     = tr->rdta->xv * zv[i] * inv_Li;*/
	  /*ersetzt durch die drei folgenden zeilen*/
	 
          /*dlnLidlz[i]     = zz[i] + zv[i] + sumcptr[i];*/
          /*t1[i]     = ((zv[i] + sumcptr[i]) == 0.0) ? 1.0 : zz[i] / dlnLidlz[i];
          t2[i]     = ((zz[i] + sumcptr[i]) == 0.0) ? tr->rdta->xv : 
	  tr->rdta->xv * zv[i] / dlnLidlz[i];*/


	  /*  dlnLidlz[i]   = t1[i] + t2[i];
          dlnLdlz   += *wrptr++  * dlnLidlz[i];
          d2lnLdlz2 += *wr2ptr++ * (t1[i] + tr->rdta->xv * t2[i] - dlnLidlz[i] * dlnLidlz[i]);*/
          }
	

        if ((d2lnLdlz2 >= 0.0) && (z < zmax))
          zprev = z = 0.37 * z + 0.63;  /*  Bad curvature, shorten branch */
        else
          curvatOK = TRUE;

        } while (! curvatOK);

      if (d2lnLdlz2 < 0.0) {
        z *= exp(-dlnLdlz / d2lnLdlz2);
        if (z < zmin) z = zmin;
        if (z > 0.25 * zprev + 0.75)    /*  Limit steps toward z = 1.0 */
          z = 0.25 * zprev + 0.75;
        }

      if (z > zmax) z = zmax;

      } while ((--maxiter > 0) && (ABS(z - zprev) > zstep));

    free(abi);
    free(bci);
    free(sumci);
    free(fx1a);
    free(fx1c);
    free(fx1g);
    free(fx1t);
    free(zv);

/* printf("makenewz: %le\n", z); */

    return  z;
  } /* makenewz */


#endif



#ifdef SEV_CODE

boolean newview(tree *tr, nodeptr p)        /*  Update likelihoods at node */
  { /* newview */
    double   zq, lzq, xvlzq, zr, lzr, xvlzr;
    nodeptr  q, r, realP;
    likelivector *lp, *lq, *lr;
    homType *mlvP;
    int  i;
           
    if (p->tip) {             /*  Make sure that data are at tip */
    
      if (p->x) return TRUE;  /*  They are already there */

      if (! getxtip(p)) return FALSE; /*  They are not, so get memory */      
      
      return TRUE;
      }

/*  Internal node needs update */

    q = p->next->back;
    r = p->next->next->back;

    while ((! p->x) || (! q->x) || (! r->x)) {
      /* rekursiver aufruf */
      if (! q->x) if (! newview(tr, q))  return FALSE;
      if (! r->x) if (! newview(tr, r))  return FALSE;
      if (! p->x) if (! getxnode(p)) return FALSE;
      }




    lp = p->x->lv;

    lq = q->x->lv;
    zq = q->z;
    lzq = (zq > zmin) ? log(zq) : log(zmin);
    xvlzq = tr->rdta->xv * lzq;

    lr = r->x->lv;
    zr = r->z;
    lzr = (zr > zmin) ? log(zr) : log(zmin);
    xvlzr = tr->rdta->xv * lzr;

    { double  zzq, zvq,
	zzr, zvr, rp;
      double  fxqr, fxqy, fxqn, sumaq, sumgq, sumcq, sumtq,
              fxrr, fxry, fxrn, ki, tempi, tempj;                
      register  char *s1; 
      register  char *s2;
      register char *eqptr;
      
      homType *mlvR, *mlvQ;
      poutsa calc[EQUALITIES],calcR[EQUALITIES], *calcptr, *calcptrR;
      long ref, ref2, mexp;      
      nodeptr realQ, realR;

      ki = tr->rdta->catrat[1];
    
      /* get equality vectors of p, q, and r */
      
      realP =  tr->nodep[p->number];
    
      mlvP = realP->mlv;
      realQ =  tr->nodep[q->number];
      realR =  tr->nodep[r->number];


      eqptr = realP->equalityVector; 
      s1 = realQ->equalityVector;
      s2 = realR->equalityVector; 
                 
      zzq = exp(ki *   lzq);
      zvq = exp(ki * xvlzq);
      zzr = exp(ki *   lzr);        
      zvr = exp(ki * xvlzr); 
      
      mlvQ = realQ->mlv;
      mlvR = realR->mlv;
           	 	  
      for(i = 0; i < equalities; i++)
	{
	  
	  if(mlvQ->set)
	    {
	      calcptr = &calc[i];		 
	      fxqr = tr->rdta->freqa * mlvQ->a + tr->rdta->freqg * mlvQ->g;
	      fxqy = tr->rdta->freqc * mlvQ->c + tr->rdta->freqt * mlvQ->t;
	      fxqn = fxqr + fxqy;
	      tempi = fxqr * tr->rdta->invfreqr;
	      tempj = zvq * (tempi-fxqn) + fxqn;
	      calcptr->sumaq = zzq * (mlvQ->a - tempi) + tempj;
	      calcptr->sumgq = zzq * (mlvQ->g - tempi) + tempj;
	      tempi = fxqy * tr->rdta->invfreqy;
	      tempj = zvq * (tempi-fxqn) + fxqn;
	      calcptr->sumcq = zzq * (mlvQ->c - tempi) + tempj;
	      calcptr->sumtq = zzq * (mlvQ->t - tempi) + tempj;
	      calcptr->exp = mlvQ->exp;
	    }
	  if(mlvR->set)
	    {
	      calcptrR = &calcR[i];		 
	      fxrr = tr->rdta->freqa * mlvR->a + tr->rdta->freqg * mlvR->g;
	      fxry = tr->rdta->freqc * mlvR->c + tr->rdta->freqt * mlvR->t;
	      fxrn = fxrr + fxry;
	      tempi = fxrr * tr->rdta->invfreqr;
	      tempj = zvr * (tempi-fxrn) + fxrn;
	      calcptrR->sumaq = zzr * (mlvR->a - tempi) + tempj;
	      calcptrR->sumgq = zzr * (mlvR->g - tempi) + tempj;
	      tempi = fxry * tr->rdta->invfreqy;
	      tempj = zvr * (tempi-fxrn) + fxrn;
	      calcptrR->sumcq = zzr * (mlvR->c - tempi) + tempj;
	      calcptrR->sumtq = zzr * (mlvR->t - tempi) + tempj;	     				
	      calcptrR->exp = mlvR->exp;	    	     	
	    }

	  if(mlvR->set && mlvQ->set)
	    {
	      mlvP->a = calcptrR->sumaq * calcptr->sumaq;
	      mlvP->g = calcptrR->sumgq * calcptr->sumgq;		    
	      mlvP->c = calcptrR->sumcq * calcptr->sumcq; 
	      mlvP->t = calcptrR->sumtq * calcptr->sumtq;
	      mlvP->exp = calcptrR->exp + calcptr->exp;
	      if (mlvP->a < minlikelihood && mlvP->g < minlikelihood
		  && mlvP->c < minlikelihood && mlvP->t < minlikelihood) {
		mlvP->a   *= twotothe256;
		mlvP->g   *= twotothe256;
		mlvP->c   *= twotothe256;
		mlvP->t   *= twotothe256;
		mlvP->exp += 1;
	      }
	 
	      mlvP->set = TRUE;
	    }
	  else
	    mlvP->set = FALSE;
	  
	  mlvQ++;
	  mlvR++;
	  mlvP++;
	}


      if(r->tip && q->tip)
	{

	 

	   for (i = 0; i < tr->cdta->endsite; i++) 
	    {
	      ref = *s2++;
	      ref2 = *s1++;
	     	      	      		  	
	      if(ref2 != ref)
		{
		  *eqptr++ = -1;
		  calcptrR = &calcR[ref];
		  calcptr = &calc[ref2];
		  lp->a = calcptrR->sumaq * calcptr->sumaq;
		  lp->g = calcptrR->sumgq * calcptr->sumgq;		    
		  lp->c = calcptrR->sumcq * calcptr->sumcq; 
		  lp->t = calcptrR->sumtq * calcptr->sumtq;
		  lp->exp = calcptrR->exp + calcptr->exp;
		  if (lp->a < minlikelihood && lp->g < minlikelihood
		      && lp->c < minlikelihood && lp->t < minlikelihood) {
		    lp->a   *= twotothe256;
		    lp->g   *= twotothe256;
		    lp->c   *= twotothe256;
		    lp->t   *= twotothe256;
		    lp->exp += 1;
		  }
		  lp++;   
		}
	      else
		{
		  *eqptr++ = ref;		    		
		}
	   	      	    
	    }          
	}
      else
	{
	  	  

	 
	  
	  for (i = 0; i < tr->cdta->endsite; i++) 
	    {
	      ref = *s2++;
	      ref2 = *s1++;
	      
	      if(ref2 >= 0)
		{		  	
		  if(ref2 == ref)
		    {
		      *eqptr++ = ref;		     		  
		      goto incTip;		    		  		 		  	    		
		    }
		  calcptr = &calc[ref2];
		  sumaq = calcptr->sumaq;
		  sumgq = calcptr->sumgq;
		  sumcq = calcptr->sumcq;
		  sumtq = calcptr->sumtq;		 		  	
		  mexp = calcptr->exp;	       		  
		}	    	  	  	 	 
	      else
		{		
		  fxqr = tr->rdta->freqa * lq->a + tr->rdta->freqg * lq->g;
		  fxqy = tr->rdta->freqc * lq->c + tr->rdta->freqt * lq->t;
		  fxqn = fxqr + fxqy;
		  tempi = fxqr * tr->rdta->invfreqr;
		  tempj = zvq * (tempi-fxqn) + fxqn;
		  sumaq = zzq * (lq->a - tempi) + tempj;
		  sumgq = zzq * (lq->g - tempi) + tempj;
		  tempi = fxqy * tr->rdta->invfreqy;
		  tempj = zvq * (tempi-fxqn) + fxqn;
		  sumcq = zzq * (lq->c - tempi) + tempj;
		  sumtq = zzq * (lq->t - tempi) + tempj;
		  mexp = lq->exp;
		  lq++;
		}
	      	      
	      if(ref >= 0)
		{
		  calcptr = &calcR[ref];		 	    
		  lp->a = sumaq * calcptr->sumaq;
		  lp->g = sumgq * calcptr->sumgq;		    
		  lp->c = sumcq * calcptr->sumcq; 
		  lp->t = sumtq * calcptr->sumtq;
		  lp->exp = mexp + calcptr->exp;		  	  
		}
	      else
		{
		  fxrr = tr->rdta->freqa * lr->a + tr->rdta->freqg * lr->g;
		  fxry = tr->rdta->freqc * lr->c + tr->rdta->freqt * lr->t;
		  fxrn = fxrr + fxry;
		  tempi = fxrr * tr->rdta->invfreqr;
		  tempj = zvr * (tempi-fxrn) + fxrn;
		  lp->a = sumaq * (zzr * (lr->a - tempi) + tempj);
		  lp->g = sumgq * (zzr * (lr->g - tempi) + tempj);
		  tempi = fxry * tr->rdta->invfreqy;
		  tempj = zvr * (tempi-fxrn) + fxrn;
		  lp->c = sumcq * (zzr * (lr->c - tempi) + tempj);
		  lp->t = sumtq * (zzr * (lr->t - tempi) + tempj);
		  lp->exp = mexp + lr->exp;
		  lr++;
		}
	      
	      if (lp->a < minlikelihood && lp->g < minlikelihood
		  && lp->c < minlikelihood && lp->t < minlikelihood) {
		lp->a   *= twotothe256;
		lp->g   *= twotothe256;
		lp->c   *= twotothe256;
		lp->t   *= twotothe256;
		lp->exp += 1;
	      }
	      
	      *eqptr++ = -1;
	      lp++;
	    incTip:	 ;
	    }      
	
    
	}
    }
                 

    return TRUE;  
  } /* newview */

 /* AxML modification end*/





double evaluate (tree *tr, nodeptr p)
  { /* evaluate */
    double   sum, z, lz, xvlz,
             ki, fx1a, fx1c, fx1g, fx1t, fx1r, fx1y, fx2r, fx2y,
             suma, sumb, sumc, term;


    double   zzp, zvp;
       
    
    likelivector *lp, *lq;
    homType *mlvP, *mlvQ;
    nodeptr       q, realQ, realP;
    int           i, *wptr;
    register  char *sQ; 
    register  char *sP;
    long refQ, refP;
    memP calcP[EQUALITIES], *calcptrP;
    memQ calcQ[EQUALITIES], *calcptrQ;

    q = p->back;

    while ((! p->x) || (! q->x)) {
      if (! (p->x)) if (! newview(tr, p)) return badEval;
      if (! (q->x)) if (! newview(tr, q)) return badEval;
    }

    lp = p->x->lv;
    lq = q->x->lv;

    z = p->z;
    if (z < zmin) z = zmin;
    lz = log(z);
    xvlz = tr->rdta->xv * lz;

    ki = tr->rdta->catrat[1];

    
    zzp = exp(ki *   lz);
    zvp = exp(ki * xvlz);
      
    wptr = &(tr->cdta->aliaswgt[0]);
   
    sum = 0.0;
    
    realQ =  tr->nodep[q->number];
    realP =  tr->nodep[p->number];
    mlvP = realP->mlv;
    mlvQ = realQ->mlv;
    sQ = realQ->equalityVector;
    sP = realP->equalityVector; 
    for(i = 0; i < equalities; i++)
	{
	 
	  if(mlvP->set)
	    {
	      calcptrP = &calcP[i];		 
	      calcptrP->fx1a = tr->rdta->freqa * mlvP->a;
	      calcptrP->fx1g = tr->rdta->freqg * mlvP->g;
	      calcptrP->fx1c = tr->rdta->freqc * mlvP->c;
	      calcptrP->fx1t = tr->rdta->freqt * mlvP->t;
	      calcptrP->sumag = calcptrP->fx1a + calcptrP->fx1g;
	      calcptrP->sumct = calcptrP->fx1c + calcptrP->fx1t;
	      calcptrP->sumagct = calcptrP->sumag + calcptrP->sumct;	      
	      calcptrP->exp = mlvP->exp;
	    }
	 
	  if(mlvQ->set)
	    {
	      calcptrQ = &calcQ[i];
	      calcptrQ->fx2r = tr->rdta->freqa * mlvQ->a + tr->rdta->freqg * mlvQ->g;	     
	      calcptrQ->fx2y  = tr->rdta->freqc * mlvQ->c + tr->rdta->freqt * mlvQ->t; 	     
	      calcptrQ->sumfx2rfx2y = calcptrQ->fx2r + calcptrQ->fx2y;	      
	      calcptrQ->exp = mlvQ->exp;
	      calcptrQ->a = mlvQ->a;
	      calcptrQ->c = mlvQ->c;
	      calcptrQ->g = mlvQ->g;
	      calcptrQ->t = mlvQ->t;
	    }
	  
	  mlvQ++;	 
	  mlvP++;
	}




      for (i = 0; i < tr->cdta->endsite; i++) {
	    refQ = *sQ++;
	    refP = *sP++;

	    if(refP >= 0)
	      {
		if(refQ >= 0)
		  {
		    calcptrP = &calcP[refP];
		    calcptrQ = &calcQ[refQ];
		    suma = calcptrP->fx1a * calcptrQ->a + 
		      calcptrP->fx1c * calcptrQ->c + 
		      calcptrP->fx1g * calcptrQ->g + calcptrP->fx1t * calcptrQ->t;		 		 
		    
		    sumc = calcptrP->sumagct * calcptrQ->sumfx2rfx2y;
		    sumb = calcptrP->sumag * calcptrQ->fx2r * tr->rdta->invfreqr + calcptrP->sumct * calcptrQ->fx2y * tr->rdta->invfreqy;
		    suma -= sumb;
		    sumb -= sumc;
		    term = log(zzp * suma + zvp * sumb + sumc) + 
		      (calcptrP->exp + calcptrQ->exp)*log(minlikelihood);
		    sum += *wptr++ * term;
		    
		   
		  }
		else
		  {
		    calcptrP = &calcP[refP];
		    suma = calcptrP->fx1a * lq->a + calcptrP->fx1c * lq->c + 
		      calcptrP->fx1g * lq->g + calcptrP->fx1t * lq->t;
		    
		    fx2r = tr->rdta->freqa * lq->a + tr->rdta->freqg * lq->g;
		    fx2y = tr->rdta->freqc * lq->c + tr->rdta->freqt * lq->t;
		    
		    sumc = calcptrP->sumagct * (fx2r + fx2y);
		    sumb       = calcptrP->sumag * fx2r * tr->rdta->invfreqr + calcptrP->sumct * fx2y * tr->rdta->invfreqy;
		    suma -= sumb;
		    sumb -= sumc;
		    term = log(zzp * suma + zvp * sumb + sumc) + 
		      (calcptrP->exp + lq->exp)*log(minlikelihood);
		    sum += *wptr++ * term;
		    
		   
		    lq++;
		  }
	      }
	    else
	      {
		if(refQ >= 0)
		  {
		    calcptrQ = &calcQ[refQ];
		    fx1a = tr->rdta->freqa * lp->a;
		    fx1g = tr->rdta->freqg * lp->g;
		    fx1c = tr->rdta->freqc * lp->c;
		    fx1t = tr->rdta->freqt * lp->t;
		    suma = fx1a * calcptrQ->a + fx1c * calcptrQ->c + 
		      fx1g * calcptrQ->g + fx1t * calcptrQ->t;	  
		    fx1r = fx1a + fx1g;
		    fx1y = fx1c + fx1t;
		    sumc = (fx1r + fx1y) * calcptrQ->sumfx2rfx2y; 
		    sumb = fx1r * calcptrQ->fx2r * tr->rdta->invfreqr + fx1y * calcptrQ->fx2y * tr->rdta->invfreqy;
		    suma -= sumb;
		    sumb -= sumc;	
		    term = log(zzp * suma + zvp * sumb + sumc) + 
		      (lp->exp + calcptrQ->exp)*log(minlikelihood);		
		    sum += *wptr++ * term;	
		    
		    lp++;
		  }
		else
		  {
		    fx1a = tr->rdta->freqa * lp->a;
		    fx1g = tr->rdta->freqg * lp->g;
		    fx1c = tr->rdta->freqc * lp->c;
		    fx1t = tr->rdta->freqt * lp->t;
		    suma = fx1a * lq->a + fx1c * lq->c + fx1g * lq->g + fx1t * lq->t;
		    fx2r = tr->rdta->freqa * lq->a + tr->rdta->freqg * lq->g;
		    fx2y = tr->rdta->freqc * lq->c + tr->rdta->freqt * lq->t;
		    fx1r = fx1a + fx1g;
		    fx1y = fx1c + fx1t;
		    sumc = (fx1r + fx1y) * (fx2r + fx2y);
		    sumb = fx1r * fx2r * tr->rdta->invfreqr + fx1y * fx2y * tr->rdta->invfreqy;
		    suma -= sumb;
		    sumb -= sumc;
		    term = log(zzp * suma + zvp * sumb + sumc) + 
		      (lp->exp + lq->exp)*log(minlikelihood);
		    sum += *wptr++ * term;		
		   
		    lp++;
		    lq++;
		  }
	      }	  
        }

     	     
    tr->likelihood = sum;    
    return  sum;
  } /* evaluate */


double makenewz (tree *tr, nodeptr p, nodeptr q, double z0, int maxiter)
  { /* makenewz */
    likelivector *lp, *lq, *mlv[EQUALITIES];
    double   dlnLidlz, dlnLdlz, d2lnLdlz2, z, zprev, zstep, lz, xvlz,
             ki, suma, sumb, sumc, ab, bc, inv_Li, t1, t2,
             fx1a, fx1c, fx1g, fx1t, fx1r, fx1y, fx2r, fx2y;
    double   zzp, zvp;
    double   *wrptr, w;
    int      i, curvatOK, eqCounter, limit;
    memP   calcP[EQUALITIES], *calcptrP;
    memQ   calcQ[EQUALITIES], *calcptrQ;
    mnzMem2 mnzEQ[EQUALITIES], *mnzEqptr;
    mnzMem *abcptr, *abcstart;
    homType *mlvP, *mlvQ; 
    register  char *sQ; 
    register  char *sP;
    nodeptr realQ, realP;
    long refQ, refP;

    while ((! p->x) || (! q->x)) {
      if (! (p->x)) if (! newview(tr, p)) return badZ;
      if (! (q->x)) if (! newview(tr, q)) return badZ;
      }

    realQ =  tr->nodep[q->number];
    realP =  tr->nodep[p->number];
    mlvP = realP->mlv;
    mlvQ = realQ->mlv;
    mnzEqptr = mnzEQ;
    sQ = realQ->equalityVector;
    sP = realP->equalityVector; 
    lp = p->x->lv;
    lq = q->x->lv;
    wrptr   = &(tr->cdta->wr[0]);
    eqCounter = 0;

    { 
      unsigned scratch_size;
      scratch_size = sizeof(mnzMem) * tr->cdta->endsite;
      if ((abcstart   = (mnzMem *) malloc(scratch_size)));
      else {
        printf("ERROR: makenewz unable to obtain space for arrays\n");
        return badZ;
        }
      
      }
    abcptr = abcstart;
    z = z0;
    zprev = z;
    zstep = (1.0 - zmax) * z + zmin;
    curvatOK = FALSE;
    if (z < zmin) z = zmin;
    else if (z > zmax) z = zmax;
    lz    = log(z);
    xvlz  = tr->rdta->xv * lz;
    ki = tr->rdta->catrat[1];
    zzp = exp(ki *   lz);
    zvp = exp(ki * xvlz);
    dlnLdlz = 0.0;                 /*  = d(ln(likelihood))/d(lz) */
    d2lnLdlz2 = 0.0;               /*  = d2(ln(likelihood))/d(lz)2 */

    for(i = 0; i < equalities; i++)
	{
	  if(mlvP->set)
	    {
	      calcptrP = &calcP[i];		 
	      calcptrP->fx1a = tr->rdta->freqa * mlvP->a;
	      calcptrP->fx1g = tr->rdta->freqg * mlvP->g;
	      calcptrP->fx1c = tr->rdta->freqc * mlvP->c;
	      calcptrP->fx1t = tr->rdta->freqt * mlvP->t;
	      calcptrP->sumag = calcptrP->fx1a + calcptrP->fx1g;
	      calcptrP->sumct = calcptrP->fx1c + calcptrP->fx1t;
	      calcptrP->sumagct = calcptrP->sumag + calcptrP->sumct;	      
	    }
	  if(mlvQ->set)
	    {
	      calcptrQ = &calcQ[i];
	      calcptrQ->fx2r = tr->rdta->freqa * mlvQ->a + tr->rdta->freqg * mlvQ->g;	     
	      calcptrQ->fx2y = tr->rdta->freqc * mlvQ->c + tr->rdta->freqt * mlvQ->t; 	     
	      calcptrQ->sumfx2rfx2y =  calcptrQ->fx2r +  calcptrQ->fx2y;	     
	      calcptrQ->a = mlvQ->a;
	      calcptrQ->c = mlvQ->c;
	      calcptrQ->g = mlvQ->g;
	      calcptrQ->t = mlvQ->t;	      
	    }
	  if(mlvQ->set && mlvP->set)
	    {
	      suma = calcptrP->fx1a * calcptrQ->a + calcptrP->fx1c * 
		calcptrQ->c + calcptrP->fx1g * calcptrQ->g + calcptrP->fx1t * 
		calcptrQ->t;
	      	      	      
	      sumc = calcptrP->sumagct * calcptrQ->sumfx2rfx2y;
	      sumb = calcptrP->sumag * calcptrQ->fx2r * tr->rdta->invfreqr 
		+ calcptrP->sumct * calcptrQ->fx2y * tr->rdta->invfreqy;

	      mnzEqptr->a = suma - sumb;
	      mnzEqptr->b = sumb - sumc;
              mnzEqptr->c = sumc;
	      ab = mnzEqptr->a * zzp;
	      bc = mnzEqptr->b * zvp;	       
	      inv_Li = 1.0/(ab + bc + mnzEqptr->c);
	      t1     = ab * inv_Li;
	      t2     = tr->rdta->xv * bc * inv_Li;
	      mnzEqptr->dlnLidlz   = t1 + t2;
	      mnzEqptr->d2lnLdlz2 = t1 + tr->rdta->xv * t2 - 
		mnzEqptr->dlnLidlz * mnzEqptr->dlnLidlz;
	    }

	  mnzEqptr->w = 0;
	  mlvQ++;	 
	  mlvP++;
	  mnzEqptr++;
	}

    for (i = 0; i < tr->cdta->endsite; i++) {
      refQ = *sQ++;
      refP = *sP++;


      if(refP >= 0)
	{
	  if(refP == refQ)
	    {
	      mnzEqptr = &mnzEQ[refP];
	      mnzEqptr->w += *wrptr++;
	     
	      eqCounter++;
	    }
	  else
	    {
	      calcptrP = &calcP[refP];

	      if(refQ >= 0)
		{		  
		  calcptrQ = &calcQ[refQ];
		  suma = calcptrP->fx1a * calcptrQ->a + calcptrP->fx1c * calcptrQ->c + 
		    calcptrP->fx1g * calcptrQ->g + calcptrP->fx1t * calcptrQ->t;		  		  
		  abcptr->c = sumc = calcptrP->sumagct * calcptrQ->sumfx2rfx2y;
		  sumb       = calcptrP->sumag * calcptrQ->fx2r * tr->rdta->invfreqr + calcptrP->sumct * calcptrQ->fx2y * tr->rdta->invfreqy;
		  abcptr->a   = suma - sumb;
		  abcptr->b   = sumb - sumc;
		  abcptr->w   = *wrptr++;
		  ab     = abcptr->a * zzp;
		  bc     = abcptr->b * zvp;         
		  inv_Li = 1.0/(ab + bc + sumc);
		  t1     = ab * inv_Li;
		  t2     = tr->rdta->xv * bc * inv_Li;
		  dlnLidlz   = t1 + t2;
		  dlnLdlz   += abcptr->w  * dlnLidlz;
		  d2lnLdlz2 += abcptr->w * (t1 + tr->rdta->xv * t2 - dlnLidlz * dlnLidlz);
		  abcptr++;
		}	    
	      else
		{	     
		  suma = calcptrP->fx1a * lq->a + calcptrP->fx1c * lq->c + 
		    calcptrP->fx1g * lq->g + calcptrP->fx1t * lq->t;
		  
		  fx2r = tr->rdta->freqa * lq->a + tr->rdta->freqg * lq->g;
		  fx2y = tr->rdta->freqc * lq->c + tr->rdta->freqt * lq->t;
		  
		  abcptr->c = sumc = calcptrP->sumagct * (fx2r + fx2y);
		  sumb       = calcptrP->sumag * fx2r  * tr->rdta->invfreqr + calcptrP->sumct * fx2y * tr->rdta->invfreqy ;
		  abcptr->a   = suma - sumb;
		  abcptr->b   = sumb - sumc;
		  abcptr->w   = *wrptr++;

		  ab     = abcptr->a * zzp;
		  bc     = abcptr->b * zvp;         
		  inv_Li = 1.0/(ab + bc + sumc);
		  t1     = ab * inv_Li;
		  t2     = tr->rdta->xv * bc * inv_Li;
		  dlnLidlz   = t1 + t2;
		  dlnLdlz   += abcptr->w  * dlnLidlz;
		  d2lnLdlz2 += abcptr->w * (t1 + tr->rdta->xv * t2 - dlnLidlz * dlnLidlz);
		  lq++;	
		  abcptr++;
		}
	    }
	}
      else
	{
	  if(refQ >= 0)
	    {
	      calcptrQ = &calcQ[refQ];
	      fx1a = tr->rdta->freqa * lp->a;
	      fx1g = tr->rdta->freqg * lp->g;
	      fx1c = tr->rdta->freqc * lp->c;
	      fx1t = tr->rdta->freqt * lp->t;
	      suma = fx1a * calcptrQ->a + fx1c * calcptrQ->c + 
		fx1g * calcptrQ->g + fx1t * calcptrQ->t;	  
	      fx1r = fx1a + fx1g;
	      fx1y = fx1c + fx1t;
	      abcptr->c = sumc = (fx1r + fx1y) * calcptrQ->sumfx2rfx2y; 
	      sumb       = fx1r * calcptrQ->fx2r  * tr->rdta->invfreqr + fx1y * calcptrQ->fx2y * tr->rdta->invfreqy;
	      abcptr->a   = suma - sumb;
	      abcptr->b   = sumb - sumc;
	      abcptr->w   = *wrptr++;
	      ab     = abcptr->a * zzp;
	      bc     = abcptr->b * zvp;         
	      inv_Li = 1.0/(ab + bc + sumc);
	      t1     = ab * inv_Li;
	      t2     = tr->rdta->xv * bc * inv_Li;
	      dlnLidlz   = t1 + t2;
	      dlnLdlz   += abcptr->w  * dlnLidlz;
	      d2lnLdlz2 += abcptr->w * (t1 + tr->rdta->xv * t2 - dlnLidlz * dlnLidlz);
	      lp++;
	      abcptr++;
	    }
	  else
	    {

	      fx1a = tr->rdta->freqa * lp->a;
	      fx1g = tr->rdta->freqg * lp->g;
	      fx1c = tr->rdta->freqc * lp->c;
	      fx1t = tr->rdta->freqt * lp->t;
	      suma = fx1a * lq->a + fx1c * lq->c + fx1g * lq->g + fx1t * lq->t;
	      fx2r = tr->rdta->freqa * lq->a + tr->rdta->freqg * lq->g;
	    
	      fx2y = tr->rdta->freqc * lq->c + tr->rdta->freqt * lq->t; 
	      
	      fx1r = fx1a + fx1g;
	      fx1y = fx1c + fx1t;
	      abcptr->c = sumc = (fx1r + fx1y) * (fx2r + fx2y); 
	     
	      sumb       = fx1r * fx2r * tr->rdta->invfreqr + fx1y * fx2y * tr->rdta->invfreqy;
    
	      abcptr->a   = suma - sumb;
	      abcptr->b   = sumb - sumc;
	      abcptr->w   = *wrptr++;
	      ab     = abcptr->a * zzp;
	      bc     = abcptr->b * zvp;         
	      inv_Li = 1.0/(ab + bc + sumc);
	      t1     = ab * inv_Li;
	      t2     = tr->rdta->xv * bc * inv_Li;
	      dlnLidlz   = t1 + t2;
	      dlnLdlz   += abcptr->w  * dlnLidlz;
	      d2lnLdlz2 += abcptr->w * (t1 + tr->rdta->xv * t2 - dlnLidlz * dlnLidlz);
	      lp++;
	      lq++;
	      abcptr++;
	    }
	}
    }

    mnzEqptr = mnzEQ;

    for (i = 0; i < equalities; i++)
      {
	if(mnzEqptr->w > 0)
	  {
	     dlnLdlz   += mnzEqptr->w * mnzEqptr->dlnLidlz;
	     d2lnLdlz2 += mnzEqptr->w * mnzEqptr->d2lnLdlz2;
	  }
	mnzEqptr++;
      }

    limit = tr->cdta->endsite - eqCounter;
      
    if ((d2lnLdlz2 >= 0.0) && (z < zmax))
      zprev = z = 0.37 * z + 0.63;  /*  Bad curvature, shorten branch */
    else
      curvatOK = TRUE;
   
    if(curvatOK)
      {
	if (d2lnLdlz2 < 0.0) 
	  {
	    z *= exp(-dlnLdlz / d2lnLdlz2);
	    if (z < zmin) z = zmin;
	    if (z > 0.25 * zprev + 0.75)    /*  Limit steps toward z = 1.0 */
	      z = 0.25 * zprev + 0.75;
	  }

	if (z > zmax) z = zmax;
      }
    else
      {
	while (! curvatOK)
	  {
	    if (z < zmin) z = zmin;
	    else if (z > zmax) z = zmax;
	    
	    lz    = log(z);
	    xvlz  = tr->rdta->xv * lz;
	    
	    zzp = exp(ki *   lz);
	    zvp = exp(ki * xvlz);
	    
	    mnzEqptr = mnzEQ;
	    abcptr = abcstart;       
	    dlnLdlz = 0.0;                 /*  = d(ln(likelihood))/d(lz) */
	    d2lnLdlz2 = 0.0;               /*  = d2(ln(likelihood))/d(lz)2 */
	    
	    
	    for (i = 0; i < equalities; i++)
	      {
		if(mnzEqptr->w > 0)
		  {
		    ab     = mnzEqptr->a * zzp;
		    bc     = mnzEqptr->b * zvp;	       
		    inv_Li = 1.0/(ab + bc + mnzEqptr->c);
		    t1     = ab * inv_Li;
		    t2     = tr->rdta->xv * bc * inv_Li;
		    dlnLidlz   = t1 + t2;
		    dlnLdlz   += mnzEqptr->w  * dlnLidlz;
		    d2lnLdlz2 += mnzEqptr->w * 
		      (t1 + tr->rdta->xv * t2 - dlnLidlz * dlnLidlz);
		  }
		mnzEqptr++;
	      }
	    
	    for (i = 0; i < limit; i++) 
	      {                    
		ab     = abcptr->a * zzp;
		bc     = abcptr->b * zvp;         
		inv_Li = 1.0/(ab + bc + abcptr->c);
		t1     = ab * inv_Li;
		t2     = tr->rdta->xv * bc * inv_Li;
		dlnLidlz   = t1 + t2;
		dlnLdlz   += abcptr->w  * dlnLidlz;
		d2lnLdlz2 += abcptr->w * 
		  (t1 + tr->rdta->xv * t2 - dlnLidlz * dlnLidlz);
		abcptr++;
	      }	    
	    
	    if ((d2lnLdlz2 >= 0.0) && (z < zmax))
	      zprev = z = 0.37 * z + 0.63;  
	    /*  Bad curvature, shorten branch */
	    else
	      curvatOK = TRUE;

	  }
      }

   


     while ((--maxiter > 0) && (ABS(z - zprev) > zstep))
      {
	zprev = z;
	zstep = (1.0 - zmax) * z + zmin;
	curvatOK = FALSE;
	
	while (! curvatOK)
	  {
	    if (z < zmin) z = zmin;
	    else if (z > zmax) z = zmax;
	    
	    lz    = log(z);
	    xvlz  = tr->rdta->xv * lz;
	    
	    zzp = exp(ki *   lz);
	    zvp = exp(ki * xvlz);
	    
	    mnzEqptr = mnzEQ;
	    abcptr = abcstart;       
	    dlnLdlz = 0.0;                 /*  = d(ln(likelihood))/d(lz) */
	    d2lnLdlz2 = 0.0;               /*  = d2(ln(likelihood))/d(lz)2 */
	    
	    
	    for (i = 0; i < equalities; i++)
	      {
		if(mnzEqptr->w > 0)
		  {
		    ab     = mnzEqptr->a * zzp;
		    bc     = mnzEqptr->b * zvp;	       
		    inv_Li = 1.0/(ab + bc + mnzEqptr->c);
		    t1     = ab * inv_Li;
		    t2     = tr->rdta->xv * bc * inv_Li;
		    dlnLidlz   = t1 + t2;
		    dlnLdlz   += mnzEqptr->w  * dlnLidlz;
		    d2lnLdlz2 += mnzEqptr->w * 
		      (t1 + tr->rdta->xv * t2 - dlnLidlz * dlnLidlz);
		  }
		mnzEqptr++;
	      }
	    
	    for (i = 0; i < limit; i++) 
	      {                    
		ab     = abcptr->a * zzp;
		bc     = abcptr->b * zvp;         
		inv_Li = 1.0/(ab + bc + abcptr->c);
		t1     = ab * inv_Li;
		t2     = tr->rdta->xv * bc * inv_Li;
		dlnLidlz   = t1 + t2;
		dlnLdlz   += abcptr->w  * dlnLidlz;
		d2lnLdlz2 += abcptr->w * 
		  (t1 + tr->rdta->xv * t2 - dlnLidlz * dlnLidlz);
		abcptr++;
	      }	    
	    
	    if ((d2lnLdlz2 >= 0.0) && (z < zmax))
	      zprev = z = 0.37 * z + 0.63;  
	    /*  Bad curvature, shorten branch */
	    else
	      curvatOK = TRUE;

	  }


	if (d2lnLdlz2 < 0.0) {
	  z *= exp(-dlnLdlz / d2lnLdlz2);
	  if (z < zmin) z = zmin;
	  if (z > 0.25 * zprev + 0.75)    /*  Limit steps toward z = 1.0 */
	    z = 0.25 * zprev + 0.75;
        }
	
	if (z > zmax) z = zmax;
	
      }

     free(abcstart);
 
    return  z;
  } /* makenewz */


#endif

#ifdef CATEGORIES
boolean newview (tree *tr, nodeptr p)        /*  Update likelihoods at node */
  { /* newview */
    double   zq, lzq, xvlzq, zr, lzr, xvlzr;
    nodeptr  q, r;
    likelivector *lp, *lq, *lr;
    int  i;

    if (p->tip) {             /*  Make sure that data are at tip */
      likelivector *l;
      int           code;
      yType        *yptr;

      if (p->x) return TRUE;  /*  They are already there */

      if (! getxtip(p)) return FALSE; /*  They are not, so get memory */
      l = p->x->lv;           /*  Pointer to first likelihood vector value */
      yptr = p->tip;          /*  Pointer to first nucleotide datum */
      for (i = 0; i < tr->cdta->endsite; i++) {
        code = *yptr++;
        l->a =  code       & 1;
        l->c = (code >> 1) & 1;
        l->g = (code >> 2) & 1;
        l->t = (code >> 3) & 1;
        l->exp = 0;
        l++;
        }
      return TRUE;
      }

/*  Internal node needs update */

    q = p->next->back;
    r = p->next->next->back;

    while ((! p->x) || (! q->x) || (! r->x)) {
      if (! q->x) if (! newview(tr, q))  return FALSE;
      if (! r->x) if (! newview(tr, r))  return FALSE;
      if (! p->x) if (! getxnode(p)) return FALSE;
      }

    lp = p->x->lv;

    lq = q->x->lv;
    zq = q->z;
    lzq = (zq > zmin) ? log(zq) : log(zmin);
    xvlzq = tr->rdta->xv * lzq;

    lr = r->x->lv;
    zr = r->z;
    lzr = (zr > zmin) ? log(zr) : log(zmin);
    xvlzr = tr->rdta->xv * lzr;

    { double  zzqtable[maxcategories+1], zvqtable[maxcategories+1],
              zzrtable[maxcategories+1], zvrtable[maxcategories+1],
             *zzqptr, *zvqptr, *zzrptr, *zvrptr, *rptr;
      double  fxqr, fxqy, fxqn, sumaq, sumgq, sumcq, sumtq,
              fxrr, fxry, fxrn, ki, tempi, tempj;
      int  *cptr;


        double zzq, zvq, zzr, zvr;
        int cat;


      rptr   = &(tr->rdta->catrat[1]);
      zzqptr = &(zzqtable[1]);
      zvqptr = &(zvqtable[1]);
      zzrptr = &(zzrtable[1]);
      zvrptr = &(zvrtable[1]);


      for (i = 1; i <= tr->rdta->categs; i++) {   /* exps for each category */
        ki = *rptr++;
        *zzqptr++ = exp(ki *   lzq);
        *zvqptr++ = exp(ki * xvlzq);
        *zzrptr++ = exp(ki *   lzr);
        *zvrptr++ = exp(ki * xvlzr);
        }

      cptr = &(tr->cdta->patcat[0]);


        for (i = 0; i < tr->cdta->endsite; i++) {
          cat = *cptr++;
          zzq = zzqtable[cat];
          zvq = zvqtable[cat];
          fxqr = tr->rdta->freqa * lq->a + tr->rdta->freqg * lq->g;
          fxqy = tr->rdta->freqc * lq->c + tr->rdta->freqt * lq->t;
          fxqn = fxqr + fxqy;
          tempi = fxqr * tr->rdta->invfreqr;
          tempj = zvq * (tempi-fxqn) + fxqn;
          sumaq = zzq * (lq->a - tempi) + tempj;
          sumgq = zzq * (lq->g - tempi) + tempj;
          tempi = fxqy * tr->rdta->invfreqy;
          tempj = zvq * (tempi-fxqn) + fxqn;
          sumcq = zzq * (lq->c - tempi) + tempj;
          sumtq = zzq * (lq->t - tempi) + tempj;

          zzr = zzrtable[cat];
          zvr = zvrtable[cat];
          fxrr = tr->rdta->freqa * lr->a + tr->rdta->freqg * lr->g;
          fxry = tr->rdta->freqc * lr->c + tr->rdta->freqt * lr->t;
          fxrn = fxrr + fxry;
          tempi = fxrr * tr->rdta->invfreqr;
          tempj = zvr * (tempi-fxrn) + fxrn;
          lp->a = sumaq * (zzr * (lr->a - tempi) + tempj);
          lp->g = sumgq * (zzr * (lr->g - tempi) + tempj);
          tempi = fxry * tr->rdta->invfreqy;
          tempj = zvr * (tempi-fxrn) + fxrn;
          lp->c = sumcq * (zzr * (lr->c - tempi) + tempj);
          lp->t = sumtq * (zzr * (lr->t - tempi) + tempj);
          lp->exp = lq->exp + lr->exp;

          if (lp->a < minlikelihood && lp->g < minlikelihood
           && lp->c < minlikelihood && lp->t < minlikelihood) {
            lp->a   *= twotothe256;
            lp->g   *= twotothe256;
            lp->c   *= twotothe256;
            lp->t   *= twotothe256;
            lp->exp += 1;
            }
          lp++;
          lq++;
          lr++;
          }


      return TRUE;
      }
  } /* newview */


double evaluate (tree *tr, nodeptr p)
  { /* evaluate */
    double   sum, z, lz, xvlz,
             ki, fxpa, fxpc, fxpg, fxpt, fxpr, fxpy, fxqr, fxqy,
             suma, sumb, sumc, term;


       double   zz, zv;

    double        zztable[maxcategories+1], zvtable[maxcategories+1],
                 *zzptr, *zvptr;
    double        *rptr;
    likelivector *lp, *lq;
    nodeptr       q;
    int           cat, *cptr, i, *wptr;

    q = p->back;
    while ((! p->x) || (! q->x)) {
      if (! (p->x)) if (! newview(tr, p)) return badEval;
      if (! (q->x)) if (! newview(tr, q)) return badEval;
      }

    lp = p->x->lv;
    lq = q->x->lv;

    z = p->z;
    if (z < zmin) z = zmin;
    lz = log(z);
    xvlz = tr->rdta->xv * lz;

    rptr  = &(tr->rdta->catrat[1]);
    zzptr = &(zztable[1]);
    zvptr = &(zvtable[1]);



    for (i = 1; i <= tr->rdta->categs; i++) {
      ki = *rptr++;
      *zzptr++ = exp(ki *   lz);
      *zvptr++ = exp(ki * xvlz);
      }

    wptr = &(tr->cdta->aliaswgt[0]);
    cptr = &(tr->cdta->patcat[0]);
   
    sum = 0.0;


      for (i = 0; i < tr->cdta->endsite; i++) {
        cat  = *cptr++;
        zz   = zztable[cat];
        zv   = zvtable[cat];
        fxpa = tr->rdta->freqa * lp->a;
        fxpg = tr->rdta->freqg * lp->g;
        fxpc = tr->rdta->freqc * lp->c;
        fxpt = tr->rdta->freqt * lp->t;
        suma = fxpa * lq->a + fxpc * lq->c + fxpg * lq->g + fxpt * lq->t;
        fxqr = tr->rdta->freqa * lq->a + tr->rdta->freqg * lq->g;
        fxqy = tr->rdta->freqc * lq->c + tr->rdta->freqt * lq->t;
        fxpr = fxpa + fxpg;
        fxpy = fxpc + fxpt;
        sumc = (fxpr + fxpy) * (fxqr + fxqy);
        sumb = fxpr * fxqr * tr->rdta->invfreqr + fxpy * fxqy * tr->rdta->invfreqy;
        suma -= sumb;
        sumb -= sumc;
        term = log(zz * suma + zv * sumb + sumc) + (lp->exp + lq->exp)*log(minlikelihood);
/* printf("evaluate: %le\n", term); */
        sum += *wptr++ * term;
        
        lp++;
        lq++;
        }


    tr->likelihood = sum;
    return  sum;
  } /* evaluate */


double makenewz (tree *tr, nodeptr p, nodeptr q, double z0, int maxiter)
  { /* makenewz */
    likelivector *lp, *lq;
    double  *abi, *bci, *sumci, *abptr, *bcptr, *sumcptr;
    double   dlnLidlz, dlnLdlz, d2lnLdlz2, z, zprev, zstep, lz, xvlz,
             ki, suma, sumb, sumc, ab, bc, inv_Li, t1, t2,
             fx1a, fx1c, fx1g, fx1t, fx1r, fx1y, fx2r, fx2y;
    double   zztable[maxcategories+1], zvtable[maxcategories+1],
            *zzptr, *zvptr;
    double  *rptr, *wrptr, *wr2ptr;
    int      cat, *cptr, i, curvatOK;


    while ((! p->x) || (! q->x)) {
      if (! (p->x)) if (! newview(tr, p)) return badZ;
      if (! (q->x)) if (! newview(tr, q)) return badZ;
      }

    lp = p->x->lv;
    lq = q->x->lv;

    { unsigned scratch_size;
      scratch_size = sizeof(double) * tr->cdta->endsite;
      if ((abi   = (double *) malloc(scratch_size)) &&
          (bci   = (double *) malloc(scratch_size)) &&
          (sumci = (double *) malloc(scratch_size))) ;
      else {
        printf("ERROR: makenewz unable to obtain space for arrays\n");
        return badZ;
        }
      }
    abptr   = abi;
    bcptr   = bci;
    sumcptr = sumci;



    for (i = 0; i < tr->cdta->endsite; i++) {
      fx1a = tr->rdta->freqa * lp->a;
      fx1g = tr->rdta->freqg * lp->g;
      fx1c = tr->rdta->freqc * lp->c;
      fx1t = tr->rdta->freqt * lp->t;
      suma = fx1a * lq->a + fx1c * lq->c + fx1g * lq->g + fx1t * lq->t;
      fx2r = tr->rdta->freqa * lq->a + tr->rdta->freqg * lq->g;
      fx2y = tr->rdta->freqc * lq->c + tr->rdta->freqt * lq->t;
      fx1r = fx1a + fx1g;
      fx1y = fx1c + fx1t;
      *sumcptr++ = sumc = (fx1r + fx1y) * (fx2r + fx2y);
      sumb       = fx1r * fx2r * tr->rdta->invfreqr + fx1y * fx2y * tr->rdta->invfreqy;
      *abptr++   = suma - sumb;
      *bcptr++   = sumb - sumc;
      lp++;
      lq++;
      }

    z = z0;
    do {
      zprev = z;
      zstep = (1.0 - zmax) * z + zmin;
      curvatOK = FALSE;

      do {
        if (z < zmin) z = zmin;
        else if (z > zmax) z = zmax;

        lz    = log(z);
        xvlz  = tr->rdta->xv * lz;
        rptr  = &(tr->rdta->catrat[1]);
        zzptr = &(zztable[1]);
        zvptr = &(zvtable[1]);



        for (i = 1; i <= tr->rdta->categs; i++) {
          ki = *rptr++;
          *zzptr++ = exp(ki *   lz);
          *zvptr++ = exp(ki * xvlz);
          }

        abptr   = abi;
        bcptr   = bci;
        sumcptr = sumci;
        cptr    = &(tr->cdta->patcat[0]);
        wrptr   = &(tr->cdta->wr[0]);
        wr2ptr  = &(tr->cdta->wr2[0]);
        dlnLdlz = 0.0;                 /*  = d(ln(likelihood))/d(lz) */
        d2lnLdlz2 = 0.0;               /*  = d2(ln(likelihood))/d(lz)2 */



        for (i = 0; i < tr->cdta->endsite; i++) {
          cat    = *cptr++;              /*  ratecategory(i) */
          ab     = *abptr++ * zztable[cat];
          bc     = *bcptr++ * zvtable[cat];
          sumc   = *sumcptr++;
          inv_Li = 1.0/(ab + bc + sumc);
          t1     = ab * inv_Li;
          t2     = tr->rdta->xv * bc * inv_Li;
          dlnLidlz   = t1 + t2;
          dlnLdlz   += *wrptr++  * dlnLidlz;
          d2lnLdlz2 += *wr2ptr++ * (t1 + tr->rdta->xv * t2 - dlnLidlz * dlnLidlz);
          }

        if ((d2lnLdlz2 >= 0.0) && (z < zmax))
          zprev = z = 0.37 * z + 0.63;  /*  Bad curvature, shorten branch */
        else
          curvatOK = TRUE;

        } while (! curvatOK);

      if (d2lnLdlz2 < 0.0) {
        z *= exp(-dlnLdlz / d2lnLdlz2);
        if (z < zmin) z = zmin;
        if (z > 0.25 * zprev + 0.75)    /*  Limit steps toward z = 1.0 */
          z = 0.25 * zprev + 0.75;
        }

      if (z > zmax) z = zmax;

      } while ((--maxiter > 0) && (ABS(z - zprev) > zstep));

    free(abi);
    free(bci);
    free(sumci);

/* printf("makenewz: %le\n", z); */

    return  z;
  } /* makenewz */

#endif



boolean update (tree *tr, nodeptr p)
  { /* update */
    nodeptr  q;
    double   z0, z;

    q = p->back;
    z0 = q->z;
   
    if ((z = makenewz(tr, p, q, z0, newzpercycle)) == badZ) return FALSE;
      
    p->z = q->z = z;
    if (ABS(z - z0) > deltaz)  tr->smoothed = FALSE;
    return TRUE;
  } /* update */


boolean smooth (tree *tr, nodeptr p)
  { /* smooth */
    nodeptr  q;

    if (! update(tr, p))               return FALSE; /*  Adjust branch */
    if (! p->tip) {                                  /*  Adjust descendants */
        q = p->next;
        while (q != p) {
          if (! smooth(tr, q->back))   return FALSE;
          q = q->next;
          }

#     if ReturnSmoothedView
	if (! newview(tr, p)) return FALSE;
#     endif
      }

    return TRUE;
  } /* smooth */


boolean smoothTree (tree *tr, int maxtimes)
  { /* smoothTree */
    nodeptr  p, q;

    p = tr->start;

    while (--maxtimes >= 0) {
      tr->smoothed = TRUE;
      if (! smooth(tr, p->back))       return FALSE;
      if (! p->tip) {
        q = p->next;
        while (q != p) {
          if (! smooth(tr, q->back))   return FALSE;
          q = q->next;
          }
        }
      if (tr->smoothed)  break;
      }

    return TRUE;
  } /* smoothTree */


boolean localSmooth (tree *tr, nodeptr p, int maxtimes)
  { /* localSmooth -- Smooth branches around p */
    nodeptr  q;

    if (p->tip) return FALSE;            /* Should be an error */

    while (--maxtimes >= 0) {
      tr->smoothed = TRUE;
      q = p;
      do {
        if (! update(tr, q)) return FALSE;
        q = q->next;
        } while (q != p);
      if (tr->smoothed)  break;
      }

    tr->smoothed = FALSE;             /* Only smooth locally */
    return TRUE;
  } /* localSmooth */


void hookup (nodeptr p, nodeptr q, double z)
  { /* hookup */
    p->back = q;
    q->back = p;
    p->z = q->z = z;
  } /* hookup */


/* Insert node p into branch q <-> q->back */


boolean insert (tree *tr, nodeptr p, nodeptr q, boolean glob)
/* glob -- Smooth tree globally? */

/*                q
                 /.
             add/ .
               /  .
             pn   .
    s ---- p      .remove
             pnn  .
               \  .
             add\ .
                 \.      pn  = p->next;
                  r      pnn = p->next->next;
 */

  { /* insert */
    nodeptr  r, s;

    r = q->back;
    s = p->back;

   

    if(Thorough)
    { 
      double  zqr, zqs, zrs, lzqr, lzqs, lzrs, lzsum, lzq, lzr, lzs, lzmax;      
     
      if ((zqr = makenewz(tr, q, r, q->z, iterations)) == badZ) return FALSE; 
      
      if ((zqs = makenewz(tr, q, s, defaultz, iterations)) == badZ) return FALSE;      
       
      if ((zrs = makenewz(tr, r, s, defaultz, iterations)) == badZ) return FALSE;
      
      
      
  

      lzqr = (zqr > zmin) ? log(zqr) : log(zmin); 
      lzqs = (zqs > zmin) ? log(zqs) : log(zmin);
      lzrs = (zrs > zmin) ? log(zrs) : log(zmin);
      lzsum = 0.5 * (lzqr + lzqs + lzrs);

      lzq = lzsum - lzrs;
      lzr = lzsum - lzqs;
      lzs = lzsum - lzqr;
      lzmax = log(zmax);

      if      (lzq > lzmax) {lzq = lzmax; lzr = lzqr; lzs = lzqs;} 
      else if (lzr > lzmax) {lzr = lzmax; lzq = lzqr; lzs = lzrs;}
      else if (lzs > lzmax) {lzs = lzmax; lzq = lzqs; lzr = lzrs;}
      
     
      hookup(p->next,       q, exp(lzq));
      hookup(p->next->next, r, exp(lzr));
      hookup(p,             s, exp(lzs));       
    
      }
    else
      { 
	double  z;
	z = sqrt(q->z);
	hookup(p->next,       q, z);
	hookup(p->next->next, r, z);
      }

   
    
    if (! newview(tr, p)) return FALSE;
    
   
    tr->opt_level = 0;

    if(Thorough)
    {
      if (! localSmooth(tr, p, smoothings)) return FALSE;	
    } 
               
    return  TRUE;
  } /* insert */


nodeptr  removeNode (tree *tr, nodeptr p)

/*                q
                 .|
          remove. |
               .  |
             pn   |
    s ---- p      |add
             pnn  |
               .  |
          remove. |
                 .|      pn  = p->next;
                  r      pnn = p->next->next;
 */

    /* remove p and return where it was */
  { /* removeNode */
    double   zqr;
    nodeptr  q, r;

    q = p->next->back;
    r = p->next->next->back;
    zqr = q->z * r->z;

    if ((zqr = makenewz(tr, q, r, zqr, iterations)) == badZ) return (node *) NULL;      

    hookup(q, r, zqr);

    p->next->next->back = p->next->back = (node *) NULL;
    return  q;
  } /* removeNode */


boolean initrav (tree *tr, nodeptr p)
  { /* initrav */
    nodeptr  q;

    if (! p->tip) {
      q = p->next;
      do {
        if (! initrav(tr, q->back))  return FALSE;
        q = q->next;
        } while (q != p);

      if (! newview(tr, p)) return FALSE;

      }

    return TRUE;
  } /* initrav */


nodeptr buildNewTip (tree *tr, nodeptr p)
  { /* buildNewTip */
    nodeptr  q;

    q = tr->nodep[(tr->nextnode)++];
    hookup(p, q, defaultz);
    return  q;
  } /* buildNewTip */








boolean readKeyValue (char *string, char *key, char *format, void *value)
  { /* readKeyValue */

    if (! (string = strstr(string, key)))  return FALSE;
    string += strlen(key);
    if (! (string = strchr(string, '=')))  return FALSE;
    string++;
    return  sscanf(string, format, value);  /* 1 if read, otherwise 0 */
  } /* readKeyValue */





void cacheZ (tree *tr)
  { /* cacheZ */
    nodeptr  p;
    int  nodes;

    nodes = tr->mxtips  +  3 * (tr->mxtips - 2);
    p = tr->nodep[1];
    while (nodes-- > 0) {p->z0 = p->z; p++;}
  } /* cacheZ */


void restoreZ (tree *tr)
  { /* restoreZ */
    nodeptr  p;
    int  nodes;

    nodes = tr->mxtips  +  3 * (tr->mxtips - 2);
    p = tr->nodep[1];
    while (nodes-- > 0) {p->z = p->z0; p++;}
  } /* restoreZ */




static int doublecompare(const void *p1, const void *p2)
{
  double i = *((double *)p1);
  double j = *((double *)p2);
  
  if (i > j)
    return (1);
  if (i < j)
    return (-1);
  return (0);
}































node * findAnyTip(nodeptr p)
  { /* findAnyTip */
    return  p->tip ? p : findAnyTip(p->next->back);
  } /* findAnyTip */













double bestThis;



boolean testInsertParsimony (tree *tr, nodeptr p, nodeptr q)
  { /* testInsert */
    double  qz;
    nodeptr  r;
       
    r = q->back;             /* Save original connection */
    qz = q->z;

   
    if (! insert(tr, p, q, FALSE)) return FALSE;
    if (evaluate(tr, p->next->next) == badEval) return FALSE;    
    insertList(tr);
    if(Multiple && tr->likelihood > bestThis)
      {     
	bestThis = tr->likelihood;
	treeString(bestTreeStr, tr, tr->start->back, 1);
      }
 
    hookup(q, r, qz);
    p->next->next->back = p->next->back = (nodeptr) NULL;
    				            
    restoreZ(tr);           
    return TRUE;
  } /* testInsert */









void addTraverseRec(tree *tr, nodeptr p, nodeptr q,
                 int mintrav, int maxtrav)
{ /* addTraverse */

 

 
  if (--mintrav <= 0) {              
    if (! testInsertParsimony(tr, p, q))  return;
    
  }
  
  if ((! q->tip) && (--maxtrav > 0)) {    /* Continue traverse? */
    addTraverseRec(tr, p, q->next->back,
		   mintrav, maxtrav);
    addTraverseRec(tr, p, q->next->next->back,
		   mintrav, maxtrav);    
  }

} 





int  rearrangeParsimony(tree *tr, nodeptr p, int mintrav, int maxtrav)
    /* rearranges the tree, globally or locally */
  { /* rearrange */
    double   p1z, p2z, q1z, q2z;
    nodeptr  p1, p2, q, q1, q2;
    int      mintrav2;

   
    if (maxtrav < 1 || mintrav > maxtrav)  return 1;

/* Moving subtree forward in tree. */
    
    if (! p->tip) {
     

      p1 = p->next->back;
      p2 = p->next->next->back;
      if (! p1->tip || ! p2->tip) {
        p1z = p1->z;
        p2z = p2->z;
        if (! removeNode(tr, p)) return badRear;
        cacheZ(tr);
        if (! p1->tip) {
           addTraverseRec(tr, p, p1->next->back,
			  mintrav, maxtrav);
         
	   addTraverseRec(tr, p, p1->next->next->back,
				     mintrav, maxtrav);          
          }

        if (! p2->tip) {
          addTraverseRec(tr, p, p2->next->back,
                                  mintrav, maxtrav);
          addTraverseRec(tr, p, p2->next->next->back,
                                  mintrav, maxtrav);
          
          }

        hookup(p->next,       p1, p1z);  /*  Restore original tree */
        hookup(p->next->next, p2, p2z);
	if (! (initrav(tr, tr->start)
	       && initrav(tr, tr->start->back))) return badRear;	  
        }
      }   /* if (! p->tip) */

/* Moving subtree backward in tree.  Minimum move is 2 to avoid duplicates */

    q = p->back;

    

    if (! q->tip && maxtrav > 1) {
      q1 = q->next->back;
      q2 = q->next->next->back;
      if (! q1->tip && (!q1->next->back->tip || !q1->next->next->back->tip) ||
          ! q2->tip && (!q2->next->back->tip || !q2->next->next->back->tip)) {
        q1z = q1->z;
        q2z = q2->z;
        if (! removeNode(tr, q)) return badRear;
        cacheZ(tr);
        mintrav2 = mintrav > 2 ? mintrav : 2;

        if (! q1->tip) {
          addTraverseRec(tr, q, q1->next->back,
                                  mintrav2 , maxtrav);
          addTraverseRec(tr, q, q1->next->next->back,
                                  mintrav2 , maxtrav);
         
          }

        if (! q2->tip) {
         addTraverseRec(tr, q, q2->next->back,
                                  mintrav2 , maxtrav);
          addTraverseRec(tr, q, q2->next->next->back,
                                  mintrav2 , maxtrav);
          
          }

        hookup(q->next,       q1, q1z);  /*  Restore original tree */
        hookup(q->next->next, q2, q2z);
	if (! (initrav(tr, tr->start)
	       && initrav(tr, tr->start->back))) return badRear;	  
        }
      }   /* if (! q->tip && maxtrav > 1) */

    return  1;
  } 


void buildTree(tree *tr, int pn, int qn)
{
  nodeptr p, q, r;
  int diff;

  p = tr->nodep[pn];
  q = tr->nodep[qn];
  r = q->back;
  /*qz = q->z;*/

  if (! removeNode(tr, p)) fprintf(stderr,"cannot remove\n");
  else fprintf(stderr,"removed\n");
  cacheZ(tr); 
  if (! insert(tr, p, q, FALSE)) fprintf(stderr,"cannot insert\n");
  else fprintf(stderr,"inserted \n");
  if (evaluate(tr, p->next->next) == badEval) fprintf(stderr,"cannot evaluate\n");
  fprintf(stderr, "build tree lh: %f \n", tr->likelihood);
  /*  hookup(q, r, qz);
      restoreZ(tr);*/
  
} 


int treeOptimizeCounter = 0;


void treeOptimize(tree *tr, int total, int myoffset, int fraction, int maxtrav, int changeTree, int changeFraction, 
		  int changeRearr, int mintrav)
{
  nodeptr  p;
  int i;
 
  
  if (maxtrav > tr->ntips - 3)  maxtrav = tr->ntips - 3;
          
  tr->opt_level = 0;
  
  bestTreeLh = tr->likelihood;
  treeString(bestTreeStr, tr, tr->start->back, 1);


  
  if(treeOptimizeCounter < 5)
    {
      mintrav = 1;
      maxtrav = 10;
    }
  


  
  for(i = 2; i < (2 * tr->mxtips - 1); i++)
    { 
      
      if(Multiple && str_readTreeLikelihood(bestTreeStr) > bestTreeLh)
	{	
	  
	  str_treeReadLen(bestTreeStr, tr);	 
	  /*treeString(bestTreeStr, tr, tr->start->back, 1);	  
	    if(treeOptimizeCounter < 5)
	    bestTreeLh = tr->likelihood;*/
	}
      
      bestThis = bestTreeLh;
      
      rearrangeParsimony(tr, tr->nodep[i], mintrav, maxtrav); 
    }

  treeOptimizeCounter++;
  


  return;
}
















/*===========  This is a problem if tr->start->back is a tip!  ===========*/
/*  All routines should be contrived so that tr->start->back is not a tip */

void initTreeString(char *treestr, tree *tr, int form)
{
   (void) sprintf(treestr, "[&&%s: version = '%s'",
                                 programName, programVersion);
   while (*treestr) treestr++;

   (void) sprintf(treestr, ", %s = %15.13g",
		  likelihood_key, unlikely);
   while (*treestr) treestr++;
   
   (void) sprintf(treestr, ", %s = %d", ntaxa_key, tr->mxtips);
   while (*treestr) treestr++;
   
   (void) sprintf(treestr,", %s = %d", opt_level_key, 0);
   while (*treestr) treestr++;
   
   (void) sprintf(treestr, ", %s = %d", smoothed_key, 0);
   while (*treestr) treestr++;
   
   (void) sprintf(treestr, "]%s", form == treeProlog ? ", " : " ");
   while (*treestr) treestr++;
}


char *treeString (char *treestr, tree *tr, nodeptr p, int form)
    /* write string with representation of tree */
    /*   form == 1 -> Newick tree */
    /*   form == 2 -> Prolog fact */
    /*   form == 3 -> PHYLIP tree */
  { /* treeString */
    double  x, z;
    char  *nameptr;
    int    c;

    if (p == tr->start->back) {
      if (form != treePHYLIP) {
        if (form == treeProlog) {
          (void) sprintf(treestr, "phylip_tree(");
          while (*treestr) treestr++;            /* move pointer to null */
          }

        (void) sprintf(treestr, "[&&%s: version = '%s'",
                                 programName, programVersion);
        while (*treestr) treestr++;

        (void) sprintf(treestr, ", %s = %15.13g",
                                 likelihood_key, tr->likelihood);
        while (*treestr) treestr++;

        (void) sprintf(treestr, ", %s = %d", ntaxa_key, tr->ntips);
        while (*treestr) treestr++;

        (void) sprintf(treestr,", %s = %d", opt_level_key, tr->opt_level);
        while (*treestr) treestr++;

        (void) sprintf(treestr, ", %s = %d", smoothed_key, tr->smoothed);
        while (*treestr) treestr++;

        (void) sprintf(treestr, "]%s", form == treeProlog ? ", " : " ");
        while (*treestr) treestr++;
        }
    }

    if (p->tip) {
      if (form != treePHYLIP) *treestr++ = '\'';
      nameptr = p->name;
      while (c = *nameptr++) {
        if (form != treePHYLIP) {if (c == '\'') *treestr++ = '\'';}
        else if (c == ' ') {c = '_';}
        *treestr++ = c;
        }
      if (form != treePHYLIP) *treestr++ = '\'';
    }

    else {
      *treestr++ = '(';
      treestr = treeString(treestr, tr, p->next->back, form);
      *treestr++ = ',';
      treestr = treeString(treestr, tr, p->next->next->back, form);
      if (p == tr->start->back) {
        *treestr++ = ',';
        treestr = treeString(treestr, tr, p->back, form);
        }
      *treestr++ = ')';
    }

    if (p == tr->start->back) {
      (void) sprintf(treestr, ":0.0%s\n", (form != treeProlog) ? ";" : ").");
      }
    else {
      z = p->z;
      if (z < zmin) z = zmin;
      x = -log(z) * tr->rdta->fracchange;
      (void) sprintf(treestr, ": %8.6f", x);  /* prolog needs the space */
      }

    while (*treestr) treestr++;     /* move pointer up to null termination */
    return  treestr;
  } /* treeString */


void treeNexus (tree *tr, nodeptr p, FILE *out)
    /* write string with representation of tree */
    /*   form == 1 -> Newick tree */
    /*   form == 2 -> Prolog fact */
    /*   form == 3 -> PHYLIP tree */
  { /* treeString */
    double  x, z;
    char  *nameptr;
    int    c;    

    if (p->tip) {      
      fprintf(out,"%d", p->number);
    }

    else {
      fprintf(out,"(");
      treeNexus(tr, p->next->back, out);
      fprintf(out,",");
      treeNexus(tr, p->next->next->back, out);
      if (p == tr->start->back) {
        fprintf(out,",");
        treeNexus(tr, p->back, out);
        }
      fprintf(out, ")");;
    }

    if (p == tr->start->back) {
      fprintf(out,";\n");
      }
    else {
    }

   
    return;
  } /* treeString */



void treeOut (FILE *treefile, tree *tr, int form)
    /* write out file with representation of final tree */
  { /* treeOut */
    int    c;
    char  *cptr, *treestr;

    treestr = (char *) malloc((tr->ntips * (nmlngth+32)) + 256);
    if (! treestr) {
      fprintf(stderr, "treeOut: malloc failure\n");
      exit(1);
      }

    (void) treeString(treestr, tr, tr->start->back, form);
    cptr = treestr;
    while (c = *cptr++) putc(c, treefile);

    fprintf(stderr, "%s\n", treestr);
    printf("%s\n", treestr);

    free(treestr);
  } /* treeOut */


/*=======================================================================*/
/*                         Read a tree from a file                       */
/*=======================================================================*/


/*  1.0.A  Processing of quotation marks in comment removed
 */

int treeFinishCom (FILE *fp, char **strp)
  { /* treeFinishCom */
    int  ch;

    while ((ch = getc(fp)) != EOF && ch != ']') {
      if (strp != NULL) *(*strp)++ = ch;    /* save character  */
      if (ch == '[') {                      /* nested comment; find its end */
        if ((ch = treeFinishCom(fp, strp)) == EOF)  break;
        if (strp != NULL) *(*strp)++ = ch;  /* save closing ]  */
        }
      }

    if (strp != NULL) **strp = '\0';        /* terminate string  */
    return  ch;
  } /* treeFinishCom */


int treeGetCh (FILE *fp)         /* get next nonblank, noncomment character */
  { /* treeGetCh */
    int  ch;

    while ((ch = getc(fp)) != EOF) {
      if (whitechar(ch)) ;
      else if (ch == '[') {                   /* comment; find its end */
        if ((ch = treeFinishCom(fp, (char **) NULL)) == EOF)  break;
        }
      else  break;
      }

    return  ch;
  } /* treeGetCh */


boolean  treeLabelEnd (int ch)
  { /* treeLabelEnd */
    switch (ch) {
        case EOF:  case '\0':  case '\t':  case '\n':  case ' ':
        case ':':  case ',':   case '(':   case ')':   case '[':
        case ';':
          return TRUE;
        default:
          break;
        }
    return FALSE;
  } /* treeLabelEnd */


boolean  treeGetLabel (FILE *fp, char *lblPtr, int maxlen)
  { /* treeGetLabel */
    int      ch;
    boolean  done, quoted, lblfound;

    if (--maxlen < 0) lblPtr = (char *) NULL;  /* reserves space for '\0' */
    else if (lblPtr == NULL) maxlen = 0;

    ch = getc(fp);
    done = treeLabelEnd(ch);

    lblfound = ! done;
    quoted = (ch == '\'');
    if (quoted && ! done) {ch = getc(fp); done = (ch == EOF);}

    while (! done) {
      if (quoted) {
        if (ch == '\'') {ch = getc(fp); if (ch != '\'') break;}
        }

      else if (treeLabelEnd(ch)) break;

      else if (ch == '_') ch = ' ';  /* unquoted _ goes to space */

      if (--maxlen >= 0) *lblPtr++ = ch;
      ch = getc(fp);
      if (ch == EOF) break;
      }

    if (ch != EOF)  (void) ungetc(ch, fp);

    if (lblPtr != NULL) *lblPtr = '\0';

    return lblfound;
  } /* treeGetLabel */


boolean  treeFlushLabel (FILE *fp)
  { /* treeFlushLabel */
    return  treeGetLabel(fp, (char *) NULL, (int) 0);
  } /* treeFlushLabel */


int  treeFindTipByLabel (char  *str, tree *tr)
                     /*  str -- label string pointer */
  { /* treeFindTipByLabel */
    nodeptr  q;
    char    *nameptr;
    int      ch, i, n;
    boolean  found;

    for (n = 1; n <= tr->mxtips; n++) {
      q = tr->nodep[n];
      if (! (q->back)) {          /*  Only consider unused tips */
        i = 0;
        nameptr = q->name;
        while ((found = (str[i++] == (ch = *nameptr++))) && ch) ;
        if (found) return n;
        }
      }

    printf("ERROR: Cannot find tree species: %s\n", str);

    return  0;
  } /* treeFindTipByLabel */


int  treeFindTipName (FILE *fp, tree *tr)
  { /* treeFindTipName */
    char    *nameptr, str[nmlngth+2];
    int      n;

    if (tr->prelabeled) {
      if (treeGetLabel(fp, str, nmlngth+2))
        n = treeFindTipByLabel(str, tr);
      else
        n = 0;
      }

    else if (tr->ntips < tr->mxtips) {
      n = tr->ntips + 1;
      nameptr = tr->nodep[n]->name;
      if (! treeGetLabel(fp, nameptr, nmlngth+1)) n = 0;
      }

    else {
      n = 0;
      }

    return  n;
  } /* treeFindTipName */


void  treeEchoContext (FILE *fp1, FILE *fp2, int n)
 { /* treeEchoContext */
   int      ch;
   boolean  waswhite;

   waswhite = TRUE;

   while (n > 0 && ((ch = getc(fp1)) != EOF)) {
     if (whitechar(ch)) {
       ch = waswhite ? '\0' : ' ';
       waswhite = TRUE;
       }
     else {
       waswhite = FALSE;
       }

     if (ch > '\0') {putc(ch, fp2); n--;}
     }
 } /* treeEchoContext */


boolean treeProcessLength (FILE *fp, double *dptr)
  { /* treeProcessLength */
    int  ch;

    if ((ch = treeGetCh(fp)) == EOF)  return FALSE;    /*  Skip comments */
    (void) ungetc(ch, fp);

    if (fscanf(fp, "%lf", dptr) != 1) {
      printf("ERROR: treeProcessLength: Problem reading branch length\n");
      treeEchoContext(fp, stdout, 40);
      printf("\n");
      return  FALSE;
      }

    return  TRUE;
  } /* treeProcessLength */


boolean  treeFlushLen (FILE  *fp)
  { /* treeFlushLen */
    double  dummy;
    int     ch;

    if ((ch = treeGetCh(fp)) == ':') return treeProcessLength(fp, & dummy);

    if (ch != EOF) (void) ungetc(ch, fp);
    return TRUE;
  } /* treeFlushLen */


boolean  treeNeedCh (FILE *fp, int c1, char *where)
  { /* treeNeedCh */
    int  c2;

    if ((c2 = treeGetCh(fp)) == c1)  return TRUE;

    printf("ERROR: Expecting '%c' %s tree; found:", c1, where);
    if (c2 == EOF) {
      printf("End-of-File");
      }
    else {
      ungetc(c2, fp);
      treeEchoContext(fp, stdout, 40);
      }
    putchar('\n');
    return FALSE;
  } /* treeNeedCh */


boolean  addElementLen (FILE *fp, tree *tr, nodeptr p)
  { /* addElementLen */
    double   z, branch;
    nodeptr  q;
    int      n, ch;

    if ((ch = treeGetCh(fp)) == '(') {     /*  A new internal node */
      n = (tr->nextnode)++;
      if (n > 2*(tr->mxtips) - 2) {
        if (tr->rooted || n > 2*(tr->mxtips) - 1) {
          printf("ERROR: Too many internal nodes.  Is tree rooted?\n");
          printf("       Deepest splitting should be a trifurcation.\n");
          return FALSE;
          }
        else {
          tr->rooted = TRUE;
          }
        }
      q = tr->nodep[n];
      if (! addElementLen(fp, tr, q->next))        return FALSE;
      if (! treeNeedCh(fp, ',', "in"))             return FALSE;
      if (! addElementLen(fp, tr, q->next->next))  return FALSE;
      if (! treeNeedCh(fp, ')', "in"))             return FALSE;
      (void) treeFlushLabel(fp);
      }

    else {                               /*  A new tip */
      ungetc(ch, fp);
      if ((n = treeFindTipName(fp, tr)) <= 0)          return FALSE;
      q = tr->nodep[n];
      if (tr->start->number > n)  tr->start = q;
      (tr->ntips)++;
      }                                  /* End of tip processing */

    if (tr->userlen) {
      if (! treeNeedCh(fp, ':', "in"))             return FALSE;
      if (! treeProcessLength(fp, & branch))       return FALSE;
      z = exp(-branch / tr->rdta->fracchange);
      if (z > zmax)  z = zmax;
      hookup(p, q, z);
      }
    else {
      if (! treeFlushLen(fp))                       return FALSE;
      hookup(p, q, defaultz);
      }

    return TRUE;
  } /* addElementLen */






int saveTreeCom (char  **comstrp)
  { /* saveTreeCom */
    int  ch;
    boolean  inquote;

    inquote = FALSE;
    while ((ch = getc(INFILE)) != EOF && (inquote || ch != ']')) {
      *(*comstrp)++ = ch;                        /* save character  */
      if (ch == '[' && ! inquote) {              /* comment; find its end */
        if ((ch = saveTreeCom(comstrp)) == EOF)  break;
        *(*comstrp)++ = ch;                      /* add ] */
        }
      else if (ch == '\'') inquote = ! inquote;  /* start or end of quote */
      }

    return  ch;
  } /* saveTreeCom */


boolean processTreeCom (FILE *fp, tree *tr)
  { /* processTreeCom */
    int   text_started, functor_read, com_open;

    /*  Accept prefatory "phylip_tree(" or "pseudoNewick("  */

    functor_read = text_started = 0;
    (void) fscanf(fp, " p%nhylip_tree(%n", & text_started, & functor_read);
    if (text_started && ! functor_read) {
      (void) fscanf(fp, "seudoNewick(%n", & functor_read);
      if (! functor_read) {
        printf("Start of tree 'p...' not understood.\n");
        return FALSE;
        }
      }

    com_open = 0;
    (void) fscanf(fp, " [%n", & com_open);

    if (com_open) {                                  /* comment; read it */
      char  com[1024], *com_end;

      com_end = com;
      if (treeFinishCom(fp, & com_end) == EOF) {     /* omits enclosing []s */
        printf("Missing end of tree comment\n");
        return FALSE;
        }

      (void) readKeyValue(com, likelihood_key, "%lg",
                               (void *) &(tr->likelihood));
      (void) readKeyValue(com, opt_level_key,  "%d",
                               (void *) &(tr->opt_level));
      (void) readKeyValue(com, smoothed_key,   "%d",
                               (void *) &(tr->smoothed));

      if (functor_read) (void) fscanf(fp, " ,");   /* remove trailing comma */
      }

    return (functor_read > 0);
  } /* processTreeCom */


nodeptr uprootTree (tree *tr, nodeptr p)
  { /* uprootTree */
    nodeptr  q, r, s, start;
    int      n;

    if (p->tip || p->back) {
      printf("ERROR: Unable to uproot tree.\n");
      printf("       Inappropriate node marked for removal.\n");
      return (nodeptr) NULL;
      }

    n = --(tr->nextnode);               /* last internal node added */
    if (n != tr->mxtips + tr->ntips - 1) {
      printf("ERROR: Unable to uproot tree.  Inconsistent\n");
      printf("       number of tips and nodes for rooted tree.\n");
      return (nodeptr) NULL;
      }

    q = p->next->back;                  /* remove p from tree */
    r = p->next->next->back;
    hookup(q, r, tr->userlen ? (q->z * r->z) : defaultz);

    start = (r->tip || (! q->tip)) ? r : r->next->next->back;

    if (tr->ntips > 2 && p->number != n) {
      q = tr->nodep[n];            /* transfer last node's conections to p */
      r = q->next;
      s = q->next->next;
      hookup(p,             q->back, q->z);   /* move connections to p */
      hookup(p->next,       r->back, r->z);
      hookup(p->next->next, s->back, s->z);
      if (start->number == q->number) start = start->back->back;
      q->back = r->back = s->back = (nodeptr) NULL;
      }
    else {
      p->back = p->next->back = p->next->next->back = (nodeptr) NULL;
      }

    tr->rooted = FALSE;
    return  start;
  } /* uprootTree */


static char buffer[] = "foobar";



boolean treeReadLen (FILE *fp, tree *tr)
  { /* treeReadLen */
    nodeptr  p;
    int      i, ch;
    boolean  is_fact;

    for (i = 1; i <= tr->mxtips; i++) tr->nodep[i]->back = (node *) NULL;
    tr->start       = tr->nodep[tr->mxtips];
    tr->ntips       = 0;
    tr->nextnode    = tr->mxtips + 1;
    tr->opt_level   = 0;
   
    tr->smoothed    = FALSE;
    tr->rooted      = FALSE;

    is_fact = processTreeCom(fp, tr);

    p = tr->nodep[(tr->nextnode)++];
    if (! treeNeedCh(fp, '(', "at start of"))       return FALSE;
    if (! addElementLen(fp, tr, p))                 return FALSE;
    if (! treeNeedCh(fp, ',', "in"))                return FALSE;
    if (! addElementLen(fp, tr, p->next))           return FALSE;
    if (! tr->rooted) {
      if ((ch = treeGetCh(fp)) == ',') {        /*  An unrooted format */
        if (! addElementLen(fp, tr, p->next->next)) return FALSE;
        }
      else {                                    /*  A rooted format */
        tr->rooted = TRUE;
        if (ch != EOF)  (void) ungetc(ch, fp);
        }
      }
    else {
      p->next->next->back = (nodeptr) NULL;
      }
    if (! treeNeedCh(fp, ')', "in"))                return FALSE;
    (void) treeFlushLabel(fp);
    if (! treeFlushLen(fp))                         return FALSE;
    if (is_fact) {
      if (! treeNeedCh(fp, ')', "at end of"))       return FALSE;
      if (! treeNeedCh(fp, '.', "at end of"))       return FALSE;
      }
    else {
      if (! treeNeedCh(fp, ';', "at end of"))       return FALSE;
      }

    if (tr->rooted) {
      p->next->next->back = (nodeptr) NULL;
      tr->start = uprootTree(tr, p->next->next);
      if (! tr->start)                              return FALSE;
      }
    else {
      tr->start = p->next->next->back;  /* This is start used by treeString */
      }

    return  (initrav(tr, tr->start) && initrav(tr, tr->start->back));
  } /* treeReadLen */










/*=======================================================================*/
/*                        Read a tree from a string                      */
/*=======================================================================*/

int str_treeFinishCom (char **treestrp, char **strp)
                      /* treestrp -- tree string pointer */
                      /* strp -- comment string pointer */
  { /* str_treeFinishCom */
    int  ch;
    while ( (ch=*(*treestrp)++)!='\0' && ch!=']' ) {
      if (strp != NULL) *(*strp)++ = ch;    /* save character  */
      if (ch == '[') {                      /* nested comment; find its end */
        if ((ch=str_treeFinishCom(treestrp,strp)) == '\0')  break;
        if (strp != NULL) *(*strp)++ = ch;  /* save closing ]  */
        }
      }
    if (strp != NULL) **strp = '\0';        /* terminate string  */
    return  ch;
  } /* str_treeFinishCom */


int str_treeGetCh (char **treestrp)
    /* get next nonblank, noncomment character */
  { /* str_treeGetCh */
    int  ch;
    while ((ch=*(*treestrp)++) != '\0') {
      if (whitechar(ch)) ;
      else if (ch == '[') {                  /* comment; find its end */
        if ((ch = str_treeFinishCom(treestrp,(char**)NULL)) == '\0')  break;
        }
      else  break;
      }

    return  ch;
  } /* str_treeGetCh */




boolean  str_treeGetLabel (char **treestrp, char *lblPtr, int maxlen)
  { /* str_treeGetLabel */
    int      ch;
    boolean  done, quoted, lblfound;


    if (--maxlen < 0)
      lblPtr = (char*)NULL;  /* reserves space for '\0' */
    else if(lblPtr == NULL)
      maxlen = 0;

    ch = *(*treestrp)++;
    done = treeLabelEnd(ch);

    lblfound = !done;
    quoted = (ch == '\'');
    if (quoted && ! done) {
      ch = *(*treestrp)++;
      done = (ch == '\0');
    }

    while(!done) {
      if (quoted) {
        if (ch == '\'') {
          ch = *(*treestrp)++;
          if (ch != '\'') break;
        }
      }
      else if (treeLabelEnd(ch))
        break;
      else if (ch == '_')
        ch = ' ';
      if (--maxlen >= 0) *lblPtr++ = ch;
      ch = *(*treestrp)++;
      if (ch == '\0') break;
    }

    (*treestrp)--;

    if (lblPtr != NULL) *lblPtr = '\0';

    return lblfound;
  } /* str_treeGetLabel */


boolean  str_treeFlushLabel (char **treestrp)
  { /* str_treeFlushLabel */
    return  str_treeGetLabel(treestrp, (char*)NULL, (int)0);
  } /* str_treeFlushLabel */


int  str_treeFindTipName (char **treestrp, tree *tr)         /*DKB-orig*/
/*int  str_treeFindTipName (char **treestrp, tree *tr, int ch)*/   /*DKB-change*/
  { /* str_treeFindTipName */
    nodeptr  q;
    char    *nameptr, str[nmlngth+2];
    int      i, n;

    if (tr->prelabeled) {
      if (str_treeGetLabel(treestrp, str, nmlngth+2)) {
        n = treeFindTipByLabel(str, tr);
      }
      else
        n = 0;
      }

    else if (tr->ntips < tr->mxtips) {
      n = tr->ntips + 1;
      nameptr = tr->nodep[n]->name;
      if (! str_treeGetLabel(treestrp, nameptr, nmlngth+1)) n = 0;
      }

    else {
      n = 0;
      }

    return  n;
  } /* str_treeFindTipName */


boolean str_treeProcessLength (char **treestrp, double *dptr)
  { /* str_treeProcessLength */
    int     used;

    if(!str_treeGetCh(treestrp))  return FALSE;    /*  Skip comments */
    (*treestrp)--;

    if (sscanf(*treestrp, "%lf%n", dptr, &used) != 1) {
      printf("ERROR: str_treeProcessLength: Problem reading branch length\n");
      printf("%40s\n", *treestrp);
      *dptr = 0.0;
      return FALSE;
      }
    else {
      *treestrp += used;
      }

    return  TRUE;
  } /* str_treeProcessLength */


boolean  str_treeFlushLen (char **treestrp)
  { /* str_treeFlushLen */
    int  ch;
    double  x;

    if ((ch = str_treeGetCh(treestrp)) == ':')
    /*return str_treeProcessLength(treestrp, (double*)NULL);*/   /*DKB-orig*/
      return str_treeProcessLength(treestrp, &x);              /*DKB-change*/
    else {
      (*treestrp)--;
      return TRUE;
      }
  } /* str_treeFlushLen */


boolean  str_treeNeedCh (char **treestrp, int c1, char *where)
  { /* str_treeNeedCh */
    int  c2, i;

    if ((c2 = str_treeGetCh(treestrp)) == c1)  return TRUE;

    printf("ERROR: Missing '%c' %s tree; ", c1, where);
    if (c2 == '\0') 
      printf("end-of-string");
    else {
      putchar('"');
      for (i = 24; i-- && (c2 != '\0'); c2 = *(*treestrp)++)  putchar(c2);
      putchar('"');
      }

    printf(" found instead\n");
    return FALSE;
  } /* str_treeNeedCh */


boolean str_processTreeCom(tree *tr, char **treestrp)
  { /* str_processTreeCom */
    char  *com, *com_end;
    int  text_started, functor_read, com_open;

    com = *treestrp;

    /* Comment must begin with either "phylip_tree" or "pseudoNewick".
     * If it is neither, return FALSE. */
    functor_read = text_started = 0;
    sscanf(com, " p%nhylip_tree(%n", &text_started, &functor_read);
    if (functor_read) {
      com += functor_read;
    }
    else if (text_started) {
      com += text_started;
      sscanf(com, "seudoNewick(%n", &functor_read);
      if (! functor_read) {
        printf("Start of tree 'p...' not understood.\n");
        return  FALSE;
      }
      else {
        com += functor_read;
      }
    }

    /* Find opening bracket of comment */
    com_open = 0;
    sscanf(com, " [%n", &com_open);
    com += com_open;

    /* Read comment. */
    if (com_open) {
      if (!(com_end = strchr(com, ']'))) {
        printf("Missing end of tree comment.\n");
        return  FALSE;
      }
      *com_end = 0;
      (void)readKeyValue(com,likelihood_key,"%lg",(void*)&(tr->likelihood));
      (void)readKeyValue(com,opt_level_key, "%d", (void*)&(tr->opt_level));
      (void)readKeyValue(com,smoothed_key,  "%d", (void*)&(tr->smoothed));
      *com_end = ']';
      com_end++;

      /* Remove trailing comma, and return addr of next char in treestp */
      if (functor_read) {
        text_started = 0;
        sscanf(com_end, " ,%n", & text_started);
        com_end += text_started;
      }
      *treestrp = com_end;
    }
    return (functor_read > 0);
  } /* str_processTreeCom */

boolean str_processTreeComMerge(int *ntaxa, char **treestrp)
  { /* str_processTreeCom */
    char  *com, *com_end;
    int  text_started, functor_read, com_open;

    com = *treestrp;

    /* Comment must begin with either "phylip_tree" or "pseudoNewick".
     * If it is neither, return FALSE. */
    functor_read = text_started = 0;
    sscanf(com, " p%nhylip_tree(%n", &text_started, &functor_read);
    if (functor_read) {
      com += functor_read;
    }
    else if (text_started) {
      com += text_started;
      sscanf(com, "seudoNewick(%n", &functor_read);
      if (! functor_read) {
        printf("Start of tree 'p...' not understood.\n");
        return  FALSE;
      }
      else {
        com += functor_read;
      }
    }

    /* Find opening bracket of comment */
    com_open = 0;
    sscanf(com, " [%n", &com_open);
    com += com_open;

    /* Read comment. */
    if (com_open) {
      if (!(com_end = strchr(com, ']'))) {
        printf("Missing end of tree comment.\n");
        return  FALSE;
      }
      *com_end = 0;
      /*(void)readKeyValue(com,likelihood_key,"%lg",(void*)&(tr->likelihood));*/
      (void)readKeyValue(com,ntaxa_key, "%d", (void *)ntaxa);   
      /*      (void)readKeyValue(com,opt_level_key, "%d", (void*)&(tr->opt_level));
	      (void)readKeyValue(com,smoothed_key,  "%d", (void*)&(tr->smoothed));*/
      *com_end = ']';
      com_end++;

      /* Remove trailing comma, and return addr of next char in treestp */
      if (functor_read) {
        text_started = 0;
        sscanf(com_end, " ,%n", & text_started);
        com_end += text_started;
      }
      *treestrp = com_end;
    }
    return (functor_read > 0);
  } /* str_processTreeCom */


boolean  str_addElementLen (char **treestrp, tree *tr, nodeptr p)
  { /* str_addElementLen */
    double   z, branch;
    nodeptr  q;
    int      n, ch;

    if ((ch = str_treeGetCh(treestrp)) == '(') { /*  A new internal node */
      n = (tr->nextnode)++;
      if (n > 2*(tr->mxtips) - 2) {
        if (tr->rooted || n > 2*(tr->mxtips) - 1) {
          printf("ERROR: too many internal nodes.  Is tree rooted?\n");
          printf("Deepest splitting should be a trifurcation.\n");
          return  FALSE;
        }
        else {
          tr->rooted = TRUE;
        }
      }
      q = tr->nodep[n];
      if (! str_addElementLen(treestrp, tr, q->next))          return FALSE;
      if (! str_treeNeedCh(treestrp, ',', "in"))               return FALSE;
      if (! str_addElementLen(treestrp, tr, q->next->next))    return FALSE;
      if (! str_treeNeedCh(treestrp, ')', "in"))               return FALSE;
    /*if (! str_treeFlushLabel(treestrp))                      return FALSE;*//*DKB-orig*/
    }

    else {                           /*  A new tip */
    /*n = str_treeFindTipName(treestrp, tr, ch);*/ /*DKB-orig*/
      if(ch=='\'') (*treestrp)--;                  /*DKB-change*/
      n = str_treeFindTipName(treestrp, tr);       /*DKB-change*/
      if (n <= 0) return FALSE;
      q = tr->nodep[n];
      if (tr->start->number > n)  tr->start = q;
      (tr->ntips)++;
    }

    /*  Master and Slave always use lengths */

    if (! str_treeNeedCh(treestrp, ':', "in"))                 return FALSE;
    if (! str_treeProcessLength(treestrp, &branch))            return FALSE;
    z = exp(-branch / tr->rdta->fracchange);
    if (z > zmax)  z = zmax;
    hookup(p, q, z);

    return  TRUE;
  } /* str_addElementLen */




boolean str_treeReadLen (char *treestr, tree *tr)
    /* read string with representation of tree */
  { /* str_treeReadLen */
    nodeptr  p;
    int  i;
    boolean  is_fact, found;

    for(i=1; i<=(tr->mxtips); i++) tr->nodep[i]->back = (node*)NULL;
    tr->start       = tr->nodep[tr->mxtips];
    tr->ntips       = 0;
    tr->nextnode    = tr->mxtips + 1;
    tr->opt_level   = 0;
    tr->smoothed    = 1;/*(myprogtype==DNAML_MASTER); DANGER !!!!!!!!!*/
    tr->rooted      = FALSE;

    is_fact = str_processTreeCom(tr,&treestr);
    p = tr->nodep[(tr->nextnode)++];

    if(!str_treeNeedCh(&treestr, '(', "at start of"))       return FALSE;
    if(!str_addElementLen(&treestr, tr, p))                 return FALSE;
    if(!str_treeNeedCh(&treestr, ',', "in"))                return FALSE;
    if(!str_addElementLen(&treestr, tr, p->next))           return FALSE;
    if(!tr->rooted) {
      if(str_treeGetCh(&treestr) == ',') {        /*  An unrooted format */
        if(!str_addElementLen(&treestr,tr,p->next->next)) return FALSE;
      }
      else {                                       /*  A rooted format */
        p->next->next->back = (nodeptr) NULL;
        tr->rooted = TRUE;
        treestr--;
      }
    }
    if(!str_treeNeedCh(&treestr, ')', "in"))                 return FALSE;
  /*if(!str_treeFlushLabel(&treestr))                        return FALSE;*//*DKB-orig*/
    if(!str_treeFlushLen(&treestr))                          return FALSE;
    if(is_fact) {
      if(!str_treeNeedCh(& treestr, ')', "at end of"))       return FALSE;
      if(!str_treeNeedCh(& treestr, '.', "at end of"))       return FALSE;
    }
    else {
      if(!str_treeNeedCh(& treestr, ';', "at end of"))       return FALSE;
    }

    if(tr->rooted)  if (! uprootTree(tr, p->next->next))     return FALSE;
    tr->start = p->next->next->back;  /* This is start used by treeString */

    return  (initrav(tr,tr->start) && initrav(tr,tr->start->back));
  } /* str_treeReadLen */








boolean treeEvaluate (tree *tr)       /* Evaluate a user tree */
  { /* treeEvaluate */
    
    if (! smoothTree(tr, smoothings)) return FALSE;      
    if (evaluate(tr, tr->start) == badEval)  return FALSE;

    return TRUE;
  } /* treeEvaluate */

















double str_readTreeLikelihood (char *treestr)
  { /* str_readTreeLikelihood */
    double lk1;
    char    *com, *com_end;
    boolean  readKeyValue();

    

    if ((com = strchr(treestr, '[')) && (com < strchr(treestr, '('))
                                     && (com_end = strchr(com, ']'))) {
      com++;
      *com_end = 0;
      if (readKeyValue(com, likelihood_key, "%lg", (void *) &(lk1))) {
        *com_end = ']';
        return lk1;
        }
      }

    fprintf(stderr, "ERROR reading likelihood in receiveTree\n");
    fprintf(stderr,"%s \n", treestr);
    return  badEval;
  } /* str_readTreeLikelihood */



















int randomInt(int n)
{
  int res =  random() % n;
  return res;
}

void makePermutation(int *perm, tree *tr)
{    
  int  i, j, k;
  int sum1 = 0, sum2 = 0;

  gettime();
  srandom((unsigned int)ttime.tv_usec);

  
  for (i = 1; i <= tr->mxtips; i++)
    {
      perm[i] = i;
      sum1 += i;
    }

  for (i = 1; i <= tr->mxtips; i++) 
    {
      k        = randomInt(tr->mxtips + 1 - i);
      j        = perm[i];
      perm[i]     = perm[i + k];
      perm[i + k] = j; 
    }

}


int get_args(int argc, char *argv[], boolean print_usage, analdef *adef)
{ /* get_args */
  int        c;
  char       *s;
  boolean    bad_opt=FALSE;
  int        i, time;
  char       buf[2048];

  run_id[0] = 0;
  workdir[0] = 0;
  seq_file[0] = 0;
  tree_file[0] = 0;
  ship_seq_data = FALSE;
  adef->restart = FALSE;

  while(!bad_opt && ((c=getopt(argc,argv,"n:w:r:m:t:s"))!=-1) ) {
    switch(c) 
      {
      case 'n':   /* Name, or ID of run */
        strcpy(run_id,optarg);	
        break;
      case 'w':   /* Working directory */
        strcpy(workdir,optarg);
        break;                 
      case 't':
	strcpy(tree_file, optarg);
	adef->restart = TRUE;
      case 's':
	break;
      case 'r':
	sscanf(optarg, "%d", &stepwidth);	
	if(stepwidth < 2) stepwidth = 2;
	break;    
      case 'm':
	sscanf(optarg, "%d", &MAX_REARRANGE);
	if(MAX_REARRANGE < 5) MAX_REARRANGE = 5;
	break;  
      default:
        bad_opt = TRUE;
        break;
    }

   


  }

 
 
 


/* Set the full path to the working directory. It's up to using functions
 * of workdir to resolve this path, which may contain instances of ./ ../
 * and //, and handle resulting errors. (N.B.: We take care not to introduce
 * double slashes because some implementations of realpath() (notably in AIX)
 * choke on them). */
  if(workdir[0]==0 || workdir[0] != '/') {
    getcwd(buf,sizeof(buf));
    if( buf[strlen(buf)-1] != '/') strcat(buf,"/");
    strcat(buf,workdir);
    if( buf[strlen(buf)-1] != '/') strcat(buf,"/");
    strcpy(workdir,buf);
  }

/* There should be one argument left, the sequence data file name. */
  if(optind==argc-1)   {  
    strcpy(seq_file,argv[argc-1]);
    }

/* Check for errors */
  if(bad_opt) {
    if(print_usage) {
      s = strrchr(argv[0],'/');
      s = (s==NULL) ? argv[0] : s+1;
      fprintf(stderr,"Usage: %s [-p<#_workers>] [-n<run_name>]\
 [-w<work_dir>] [-s] sequence_file\n",s);
      fflush(stderr);
    }
    return 0;
  }
  else {
    if(run_id[0]==0) {
      s = strrchr(seq_file,'/');
      s = (s==NULL) ? seq_file : s+1;
      strcpy(run_id,s);
    }
    return 1;
  }

  


} /* get_args */


void initList(int size)
{
  int i = 0;

  tl->trees = (listElement **)malloc(sizeof(listElement *) * size);
  for(i = 0; i < size; i++)
    {
      tl->trees[i] = (listElement *)malloc(sizeof(listElement));
    }
  tl->count = 0;
  tl->size = size;
  tl->worst = unlikely;
  tl->worst_i = 0;
  for(i = 0; i < size; i++)
    {
      tl->trees[i]->tree = (char *)malloc(treeStringLength);
      tl->trees[i]->lh = unlikely;
    }  
}


void resetList()
{
  tl->count = 0;
  tl->worst = unlikely;
  tl->worst_i = 0;  
}

void printlist()
{
  int i = 0;

  for(i = 0; i < tl->size; i++)
    fprintf(stderr,"tl: %f\n", tl->trees[i]->lh); 
}


void insertList(tree *tr)
{
  int i = 0;
  double minval = 0;

  if(tr->likelihood <= tl->worst && tl->count == tl->size) return;  
  treeString (myTreestr, tr, tr->start->back, 1);
  if(tl->count == tl->size)
    {
      strcpy(tl->trees[tl->worst_i]->tree, myTreestr);
      tl->trees[tl->worst_i]->lh = tr->likelihood;            
    }
  else
    {
      strcpy(tl->trees[tl->count]->tree, myTreestr);
      tl->trees[tl->count]->lh = tr->likelihood;        
      tl->count++;
    }
  
  tl->worst = 0;
  for( i = 0; i < tl->count; i++)
    {
      if(tl->trees[i]->lh < tl->worst)
	{
	  tl->worst = tl->trees[i]->lh;
	  tl->worst_i = i;
	}
    }

  return;

}





permutationList *permutationListInsert(double val, char *treestr, permutationList *plist, tree *tr)
{
  int i;
  permutationList *element = plist;
  permutationList *temp = (permutationList *)malloc(sizeof(permutationList));
  temp->val = val;
  temp->tree = (char *) malloc(treeStringLength);
  strcpy(temp->tree, treestr);
  temp->next =  NULL;

  if(plist == NULL)
    {
      return temp;
    }

  if(element->val < temp->val)
    {
      temp->next = element;
      return temp;
    }

  while(element->next != NULL && element->next->val > temp->val)
    {
      element = element->next;
    }   
  
  return plist;
}



permutationList *permutationListReset(permutationList *plist)
{
  permutationList *tmp;
  permutationList *element = plist;


  while(element != NULL)
    { 
      tmp = element;
      element = element->next;
      free(tmp->tree);
      free(tmp);           
    }

  return (permutationList*)NULL;
}


void printList(permutationList *plist, FILE *stream)
{
  permutationList *temp = plist;

  if(temp == NULL) 
    {
      return;
    }
 do
    {
      fprintf(stream, "%s \n", temp->tree);      
      temp = temp->next;
    }
 while(temp != NULL);  
}

void printListLh(permutationList *plist, FILE *stream)
{
  permutationList *temp = plist;

  if(temp == NULL) 
    {
      return;
    }
 do
    {
      fprintf(stream, "%f \n", temp->val);      
      temp = temp->next;
    }
 while(temp != NULL);  
}

char *getList(permutationList *plist, int k)
{
  permutationList *temp = plist;
  int c = 0;

  if(temp == NULL) 
    {
      return (char *)NULL;
    }
 do
    {
      if(c == k)
	return temp->tree;
      temp = temp->next;
      c++;
    }
 while(temp != NULL);  
 return (char *)NULL;
}










void printForMrBayes(tree *tr)
{     
  fprintf(mb,";\n");
  fprintf(mb,"END;\n");
  fprintf(mb,"begin mrbayes;\n");
  
  fprintf(mb,"usertree=");
  treeNexus(tr, tr->start->back, mb);
  fprintf(mb,"mcmc filename=%s startingtree=user;\n", run_id);
  fprintf(mb,"end\n");
  fclose(mb);
  fprintf(stderr,"MrBayes input file written to:%s\n", bayesFileName);
}

boolean buildSimpleTree (tree *tr, int ip, int iq, int ir)
  { /* buildSimpleTree */
    /* p, q and r are tips meeting at s */
    nodeptr  p, s;
    int  i;

    i = MIN(ip, iq);
    if (ir < i)  i = ir; 
    tr->start = tr->nodep[i];
    tr->ntips = 3;
    p = tr->nodep[ip];
    hookup(p, tr->nodep[iq], defaultz);
    s = buildNewTip(tr, tr->nodep[ir]);

    return insert(tr, s, p, FALSE);  /* Smoothing is local to s */
  } /* buildSimpleTree */





boolean str_treeReadLenSpecial (char *treestr, tree *tr)
    /* read string with representation of tree */
  { /* str_treeReadLen */
    nodeptr  p;
    int  i;
    boolean  is_fact, found;

    for (i = 1; i <= tr->mxtips; i++) tr->nodep[i]->back = (node *) NULL;
    tr->start       = tr->nodep[tr->mxtips];
    tr->ntips       = 0;
    tr->nextnode    = tr->mxtips + 1;
    tr->opt_level   = 0;
    tr->smoothed    = FALSE;
    tr->rooted      = FALSE;


    is_fact = str_processTreeCom(tr,&treestr);
    p = tr->nodep[(tr->nextnode)++];

    if(!str_treeNeedCh(&treestr, '(', "at start of"))       return FALSE;
    if(!str_addElementLen(&treestr, tr, p))                 return FALSE;
    if(!str_treeNeedCh(&treestr, ',', "in"))                return FALSE;
    if(!str_addElementLen(&treestr, tr, p->next))           return FALSE;
    if (! tr->rooted) 
      { 
	if(str_treeGetCh(&treestr) == ',')
	  {
	    if (! str_addElementLen(&treestr, tr, p->next->next)) return FALSE;
	  }
	else
	  {
	    tr->rooted = TRUE;
	    if(*treestr != '\n') treestr--;
	  }
      }
    else
      {
	p->next->next->back = (nodeptr) NULL;
      }
      
    if(!str_treeNeedCh(&treestr, ')', "in"))                 return FALSE;
    str_treeFlushLabel(&treestr);           
    if(!str_treeFlushLen(&treestr))                          return FALSE;
    if(is_fact) {
      if(!str_treeNeedCh(& treestr, ')', "at end of"))       return FALSE;
      if(!str_treeNeedCh(& treestr, '.', "at end of"))       return FALSE;
    }
    else 
      {
	if(!str_treeNeedCh(& treestr, ';', "at end of"))       return FALSE;
      }

    if(tr->rooted)
      {
	p->next->next->back = (nodeptr) NULL;
	tr->start =  uprootTree(tr, p->next->next);
	if(!tr->start) return FALSE;
	}
    else
      {
	tr->start = p->next->next->back;
      }

    return  (initrav(tr,tr->start) && initrav(tr,tr->start->back));
  } /* str_treeReadLen */




main (int argc, char *argv[]) 
  { /* DNA Maximum Likelihood */
    analdef      *adef;
    rawdata      *rdta;
    cruncheddata  *cdta;
    tree         *tr;    /*  current tree */   
    int i, j,  *perm, l, k;    
    FILE *permutationFile, *resultFile;
    double masterTime;    
    char permFileName[1024], resultFileName[1024];         
    int *optimizeArray;
    double lh, t;
    int impr;				 
    int rearrangementsMax, rearrangementsMin;
    char *parsimonyTree;
    int noImproveCount;
    char *outString;
    double previousLh;


    if (! (adef = (analdef *) malloc(sizeof(analdef)))) 
      {
	printf("ERROR: Unable to get memory for analysis definition\n\n");
	return 1;
      }
    if(!get_args(argc,argv,(boolean)1, adef));
    if (! (rdta = (rawdata *) malloc(sizeof(rawdata)))) 
      {
	printf("ERROR: Unable to get memory for raw DNA\n\n");
	return 1;
      }

    if (! (cdta = (cruncheddata *) malloc(sizeof(cruncheddata)))) 
      {
	printf("ERROR: Unable to get memory for crunched DNA\n\n");
	return 1;
      }

    if (tr = (tree *)     malloc(sizeof(tree)));        
    else {
      printf("ERROR: Unable to get memory for trees\n\n");
      return 1;
      }
	
    masterTime = gettime();
	  
    
    
    INFILE = fopen(seq_file, "r");
    if (! getinput(adef, rdta, cdta, tr)) return 1;
    fclose(INFILE);
    
     
    
    

    if (! makeweights(adef, rdta, cdta))                             return 1;
    if (! makevalues(rdta, cdta))                                    return 1;
    if (adef->empf && ! empiricalfreqs(rdta, cdta))                  return 1;
    reportfreqs(adef, rdta);
    if (! linkdata2tree(rdta, cdta, tr))                             return 1;
    if (! linkxarray(3, 3, cdta->endsite, & freextip, & usedxtip))   return 1;
    if (! setupnodex(tr))                                            return 1; 
   
    


#ifdef SEV_CODE
    if( ! setupEqualityVector(tr))                                    return 1;
#endif
   
    treeStringLength = (tr->mxtips * (nmlngth+32)) + 256 + (tr->mxtips * 2);   

    myTreestr = (char *) malloc(treeStringLength);    
    bestTreeStr = (char *) malloc(treeStringLength);
    perm = (int *)malloc((2 * tr->mxtips + 1) * sizeof(int));
    optimizeArray = (int *)malloc(8 * sizeof(int));
    
    tl = (treelist *)malloc(sizeof(treelist));
    initList(20);

   
    
   

    strcpy(permFileName, workdir);    
    strcpy(resultFileName, workdir);
   
    
    strcat(permFileName, "/RAxML_perm.");
    strcat(resultFileName, "/RAxML_result.");
   
    strcat(permFileName, run_id);
    strcat(resultFileName, run_id);      

   
    

   
    
    if (adef->restart) 
      {
	INFILE = fopen(tree_file, "r");	
	if (! treeReadLen(INFILE, tr))          return FALSE;
	if (! smoothTree(tr, smoothings))       return FALSE;
	if (evaluate(tr, tr->start) == badEval) return FALSE;
	lh = tr->likelihood;
	treeString(myTreestr, tr, tr->start->back, 1);
	plist = permutationListInsert(lh, myTreestr, plist, tr);	
	fclose(INFILE);
      }
    else
      {  
	t = gettime();        	    		    	   
	makePermutation(perm, tr);	    
	RAxML_parsimony(perm, treeStringLength, &parsimonyTree);	    
	str_treeReadLenSpecial(parsimonyTree ,tr);
	treeEvaluate(tr);	         	    
	free(parsimonyTree);	    	    
	lh =  tr->likelihood;
	treeString(myTreestr, tr, tr->start->back, 1);
	plist = permutationListInsert(lh, myTreestr, plist, tr);		    	   	     
	RAxML_parsimony_cleanup();	    
	permutationFile = fopen(permFileName, "w");
	printList(plist, permutationFile);
	fclose(permutationFile);	
      }
    
   
		  
    impr = 1;		  		  
    noImproveCount = 0;
    while(1)
      {



	
	t = gettime();	
	str_treeReadLen(getList(plist, 0) ,tr);
	lh = tr->likelihood;		      
	fprintf(stderr,"%f %f\n", gettime() -masterTime, lh);
	
	
	if(impr)
	  {
	    noImproveCount = 0;
	    Thorough = 0;
	    noImproveCount = 0;
	    rearrangementsMax = stepwidth;
	    rearrangementsMin = 1;		
	    resultFile = fopen(resultFileName, "w");
	    outString = myTreestr;
	    while(*outString++ != ']');
	    while(*outString != '\0')
	      {
		if(*outString != '\'')
		  fprintf(resultFile, "%c", *outString); 
		outString++;
	      }
	    fclose(resultFile);	 
	 
	    
	    optimizeArray[0] = 0;
	    optimizeArray[1] = 0;                /*myoffset*/
	    optimizeArray[2] = 0;         /*fraction*/
	    optimizeArray[3] = rearrangementsMax;/*maxtrav*/
	    optimizeArray[4] = 0;                /*changeTree*/
	    optimizeArray[5] = 0;                /*changeFraction*/				  				
	    optimizeArray[6] = 0;                /*changeRearr*/
	    optimizeArray[7] = rearrangementsMin;                /*mintrav*/				  	
	    
	    treeOptimize(tr, optimizeArray[0], optimizeArray[1], optimizeArray[2], optimizeArray[3], 
			 optimizeArray[4], optimizeArray[5], optimizeArray[6], optimizeArray[7]);   	
	  }			  			
	else
	  {
	    
	    noImproveCount++;
	    if(noImproveCount == 1)
	      { 
		Thorough = 1;
		Multiple = 0;
		rearrangementsMax = stepwidth;
		rearrangementsMin = 1; 
	      }
	   
	    if(noImproveCount > 1)
	      {   
		rearrangementsMax += stepwidth;
		rearrangementsMin += stepwidth; 
	      }
	    
	      
	    if(rearrangementsMax < MAX_REARRANGE)
	      {		 	  
		optimizeArray[0] = 0;
		optimizeArray[1] = 0;
		optimizeArray[2] = 0;         /*fraction*/
		optimizeArray[3] = rearrangementsMax;/*maxtrav*/
		optimizeArray[4] = 0;                /*changeTree*/
		optimizeArray[5] = 0;                /*changeFraction*/
		optimizeArray[6] = 0;                /*changeRearr*/			 
		optimizeArray[7] = rearrangementsMin;                /*mintrav*/  					  
		
		treeOptimize(tr, optimizeArray[0], optimizeArray[1], optimizeArray[2], optimizeArray[3], 
			     optimizeArray[4], optimizeArray[5], optimizeArray[6], optimizeArray[7]);  
	      }
	    else
	      {			    			     				  		  		 
		printForMrBayes(tr);		 
		goto cleanup;
	      }	  
	  }
	
	impr = 0;			      
	
	previousLh = lh;

		
	for(i = 0; i < tl->size; i++)
	  {	    	    	
	    str_treeReadLen(tl->trees[i]->tree ,tr);		
	    
	    smoothTree(tr, smoothings/4); 
	    evaluate(tr, tr->start);				
	    if(tr->likelihood > lh)
	      {
		impr = 1;	       
		lh = tr->likelihood;
		treeString(myTreestr, tr, tr->start->back, 1);
		plist= permutationListInsert(tr->likelihood, myTreestr, plist, tr);
	      }
	   
	  }
	
	if(impr && (!Thorough))
	  {			
	    if((-1.0 * previousLh) - (-1.0 * lh) < 1.0)
	      {
		Thorough = 1;
		Multiple = 0;
	      }
	  }
	resetList();      
      }    
    
  cleanup:   
    free(optimizeArray);
    free(myTreestr);		
    free(perm);
    free(rdta->y0);
    if(rdta->wgt != NULL)
      free(rdta->wgt);
    if(rdta->wgt2 != NULL)
      free(rdta->wgt2);
    if(rdta->sitecat != NULL)
      free(rdta->sitecat);		
    if(cdta->alias != NULL)
      free(cdta->alias);
    if(cdta->aliaswgt != NULL)
      free(cdta->aliaswgt);    
    if(cdta->patcat != NULL)
      free(cdta->patcat);
    if(cdta->patrat != NULL)
      free(cdta->patrat);
    if(cdta->wr != NULL)
      free(cdta->wr);
    if(cdta->wr2 != NULL)
      free(cdta->wr2);
    freextip = (xarray *) NULL;
    usedxtip = (xarray *) NULL;
    freeTree(tr);
    plist = permutationListReset(plist);
    
    
    masterTime = gettime() - masterTime;
    
    fprintf(stderr, "execution time: %f\n\n", masterTime);	   	   
    if(!adef->restart)
      fprintf(stderr,"Permutation Results written to: %s\n", permFileName);   	          
    fprintf(stderr,"Final Result written to: %s\n", resultFileName);  
  }
